
class ConfigManager(object):

    def __init__(self, conan_api, printer):
        self._conan_api = conan_api
        self.printer = printer

    def install(self, url, args=None):
        message = "Installing config from address %s" % url
        if args:
            message += " with args \"%s\"" % args
        self.printer.print_message(message)
        self._conan_api.config_install(url, verify_ssl=True, args=args)



import unittest

from conans.errors import ConanException

from cpt.config import ConfigManager
from cpt.printer import Printer
from cpt.test.integration.base import BaseTest
from cpt.test.unit.packager_test import MockConanAPI


class RemotesTest(unittest.TestCase):

    def setUp(self):
        self.conan_api = MockConanAPI()

    def test_valid_config(self):
        manager = ConfigManager(self.conan_api, Printer())
        manager.install('https://github.com/bincrafters/bincrafters-config.git')

    def test_valid_config_with_args(self):
        manager = ConfigManager(self.conan_api, Printer())
        manager.install('https://github.com/bincrafters/bincrafters-config.git', '-b master')


class RemotesTestRealApi(BaseTest):

    def test_valid_config(self):
        manager = ConfigManager(self.api, Printer())

        profiles = self.api.profile_list()
        self.assertEquals(len(profiles), 0)

        manager.install("https://github.com/bincrafters/bincrafters-config.git", "-b master")

        profiles = self.api.profile_list()
        self.assertGreater(len(profiles), 3)

    def test_invalid_config(self):
        manager = ConfigManager(self.api, Printer())

        profiles = self.api.profile_list()
        self.assertEquals(len(profiles), 0)

        try:
            manager.install("https://github.com/")
            self.fail("Could not accept wrong URL")
        except ConanException:
            pass



import unittest

from conans.test.utils.tools import TestClient
from cpt.test.test_client.tools import get_patched_multipackager


class PythonRequiresTest(unittest.TestCase):

    def test_python_requires(self):
        base_conanfile = """from conans import ConanFile
myvar = 123

def myfunct():
    return 234

class Pkg(ConanFile):
    pass
"""

        conanfile = """from conans import ConanFile

class Pkg(ConanFile):
    name = "pyreq"
    version = "1.0.0"
    python_requires = "pyreq_base/0.1@user/channel"

    def build(self):
        v = self.python_requires["pyreq_base"].module.myvar
        f = self.python_requires["pyreq_base"].module.myfunct()
        self.output.info("%s,%s" % (v, f))
"""

        client = TestClient()
        client.save({"conanfile_base.py": base_conanfile})
        client.run("export conanfile_base.py pyreq_base/0.1@user/channel")

        client.save({"conanfile.py": conanfile})
        mulitpackager = get_patched_multipackager(client, username="user",
                                                          channel="testing",
                                                          exclude_vcvars_precommand=True)
        mulitpackager.add({}, {})
        mulitpackager.run()
        self.assertIn("pyreq/1.0.0@user/", client.out)
        self.assertIn(": 123,234", client.out)




__version__ = '0.33.1'
NEWEST_CONAN_SUPPORTED = "1.26.000"


def get_client_version():
    from conans.model.version import Version
    from conans import __version__ as client_version
    # It is a mess comparing dev versions, lets assume that the -dev is the further release
    return Version(client_version.replace("-dev", ""))



import unittest

from conans import tools
from cpt.printer import Printer
from cpt.remotes import RemotesManager
from cpt.test.unit.packager_test import MockConanAPI


class RemotesTest(unittest.TestCase):

    def setUp(self):
        self.conan_api = MockConanAPI()

    def assert_serial_deserial(self, manager, expected):
        self.assertEquals(manager.env_vars(), expected)
        with tools.environment_append(expected):
            manager = RemotesManager(self.conan_api, Printer())
            self.assertEquals(manager.env_vars(), expected)

    def test_plain(self):
        manager = RemotesManager(self.conan_api, Printer(), remotes_input="url1, url", upload_input="url1")
        expected = {'CONAN_REMOTES': 'url1@True@remote0,url@True@remote1',
                    'CONAN_UPLOAD': 'url1@True@upload_repo'}
        self.assert_serial_deserial(manager, expected)

    def test_list(self):
        remotes_input = [("url1", True, "remote1"),
                         ("url2", False, "remote2")]
        upload_input = ("url3", True, "remote1")
        manager = RemotesManager(self.conan_api, Printer(), remotes_input=remotes_input,
                                 upload_input=upload_input)
        expected = {'CONAN_REMOTES': 'url1@True@remote1,url2@False@remote2',
                    'CONAN_UPLOAD': 'url3@True@remote1'}
        self.assert_serial_deserial(manager, expected)



import unittest

from conans import tools

from cpt.builds_generator import get_visual_builds, get_mingw_builds, get_osx_apple_clang_builds, get_linux_gcc_builds, get_linux_clang_builds
from conans.model.ref import ConanFileReference


class GeneratorsTest(unittest.TestCase):

    def test_mingw_generator(self):

        mingw_configurations = [("4.9", "x86", "dwarf2", "posix")]
        ref = ConanFileReference.loads("lib/1.0@conan/stable")
        builds = get_mingw_builds(mingw_configurations,
                                  ConanFileReference.loads("mingw_installer/1.0@conan/stable"),
                                  ["x86"], "pack:shared", ["Release", "Debug"], [None], options={},
                                  reference=ref)
        expected = [
            ({'build_type': 'Release', 'compiler.version': '4.9', 'compiler.libcxx': "libstdc++",
              'compiler': 'gcc', 'arch': 'x86', 'compiler.exception': 'dwarf2',
              'compiler.threads': 'posix'},
             {'pack:shared': True},
             {},
             {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),
            ({'compiler.version': '4.9', 'compiler': 'gcc', 'compiler.libcxx': "libstdc++",
              'build_type': 'Debug', 'compiler.exception': 'dwarf2', 'compiler.threads': 'posix',
              'arch': 'x86'},             {'pack:shared': True},
             {},
             {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),

            ({'build_type': 'Release', 'compiler.version': '4.9', 'compiler.libcxx': "libstdc++",
              'compiler': 'gcc', 'arch': 'x86', 'compiler.exception': 'dwarf2',
              'compiler.threads': 'posix'},
             {'pack:shared': False},
                {},
                {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),
            ({'compiler.version': '4.9', 'compiler': 'gcc', 'compiler.libcxx': "libstdc++",
              'build_type': 'Debug', 'compiler.exception': 'dwarf2', 'compiler.threads': 'posix',
              'arch': 'x86'},
             {'pack:shared': False},
             {},
             {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_mingw_builds(mingw_configurations, ConanFileReference.loads(
            "mingw_installer/1.0@conan/stable"), ["x86"], "pack:shared", ["Release"], ["20"], options={})
        expected = [
            ({'build_type': 'Release', 'compiler.version': '4.9', 'compiler.libcxx': "libstdc++",
              'compiler': 'gcc', 'arch': 'x86', 'compiler.exception': 'dwarf2',
              'compiler.threads': 'posix', 'compiler.cppstd': '20'},
             {'pack:shared': True},
             {},
             {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, None),
            ({'build_type': 'Release', 'compiler.version': '4.9', 'compiler.libcxx': "libstdc++",
              'compiler': 'gcc', 'arch': 'x86', 'compiler.exception': 'dwarf2',
              'compiler.threads': 'posix', 'compiler.cppstd': '20'},
             {'pack:shared': False},
             {},
             {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_mingw_builds(mingw_configurations, ConanFileReference.loads(
            "mingw_installer/1.0@conan/stable"), ["x86"], "pack:shared", ["Debug"], ["14"], options={})
        expected = [
            ({'compiler.version': '4.9', 'compiler': 'gcc', 'compiler.libcxx': "libstdc++",
              'build_type': 'Debug', 'compiler.exception': 'dwarf2', 'compiler.threads': 'posix',
              'arch': 'x86', 'compiler.cppstd': '14'},
             {'pack:shared': True},
             {},
             {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, None),
            ({'compiler.version': '4.9', 'compiler': 'gcc', 'compiler.libcxx': "libstdc++",
              'build_type': 'Debug', 'compiler.exception': 'dwarf2', 'compiler.threads': 'posix',
              'arch': 'x86', 'compiler.cppstd': '14'},
             {'pack:shared': False},
             {},
             {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_mingw_builds(mingw_configurations,
                                  ConanFileReference.loads("mingw_installer/1.0@conan/stable"),
                                  ["x86"], "pack:shared", ["Release", "Debug"], [None],
                                  options={"pack:foobar": True, "foobar:qux": "data"},
                                  reference=ref)
        expected = [
            ({'build_type': 'Release', 'compiler.version': '4.9', 'compiler.libcxx': "libstdc++",
              'compiler': 'gcc', 'arch': 'x86', 'compiler.exception': 'dwarf2',
              'compiler.threads': 'posix'},
             {'pack:shared': True, "pack:foobar": True, "foobar:qux": "data"},
             {},
             {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),
            ({'compiler.version': '4.9', 'compiler': 'gcc', 'compiler.libcxx': "libstdc++",
              'build_type': 'Debug', 'compiler.exception': 'dwarf2', 'compiler.threads': 'posix',
              'arch': 'x86'},
             {'pack:shared': True, "pack:foobar": True, "foobar:qux": "data"},
             {},
             {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),

            ({'build_type': 'Release', 'compiler.version': '4.9', 'compiler.libcxx': "libstdc++",
              'compiler': 'gcc', 'arch': 'x86', 'compiler.exception': 'dwarf2',
              'compiler.threads': 'posix'},
             {'pack:shared': False, "pack:foobar": True, "foobar:qux": "data"},
             {},
             {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),
            ({'compiler.version': '4.9', 'compiler': 'gcc', 'compiler.libcxx': "libstdc++",
              'build_type': 'Debug', 'compiler.exception': 'dwarf2', 'compiler.threads': 'posix',
              'arch': 'x86'},
             {'pack:shared': False, "pack:foobar": True, "foobar:qux": "data"},
             {},
             {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_mingw_builds(mingw_configurations,
                                  ConanFileReference.loads("mingw_installer/1.0@conan/stable"),
                                  ["x86"], None, ["Release", "Debug"], [None],
                                  options={"pack:foobar": True, "foobar:qux": "data"},
                                  reference=ref)
        expected = [
            ({'build_type': 'Release', 'compiler.version': '4.9', 'compiler.libcxx': "libstdc++",
              'compiler': 'gcc', 'arch': 'x86', 'compiler.exception': 'dwarf2',
              'compiler.threads': 'posix'},
             {"pack:foobar": True, "foobar:qux": "data"},
             {},
             {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),
            ({'compiler.version': '4.9', 'compiler': 'gcc', 'compiler.libcxx': "libstdc++",
              'build_type': 'Debug', 'compiler.exception': 'dwarf2', 'compiler.threads': 'posix',
              'arch': 'x86'},
             {"pack:foobar": True, "foobar:qux": "data"},
             {},
             {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref)]

        self.assertEquals([tuple(a) for a in builds], expected)

        mingw_configurations = [("4.9", "x86", "dwarf2", "posix")]
        ref = ConanFileReference.loads("lib/1.0@conan/stable")
        builds = get_mingw_builds(mingw_configurations,
                                  ConanFileReference.loads("mingw_installer/1.0@conan/stable"),
                                  ["x86"], "pack:shared", ["Release", "Debug"], [None], options={},
                                  reference=ref,
                                  build_all_options_values=[
                                  {'pack:shared': True, 'pack:foo': True, 'pack:bar': True},
                                  {'pack:shared': True, 'pack:foo': False, 'pack:bar': True},
                                  {'pack:shared': True, 'pack:foo': True, 'pack:bar': False},
                                  {'pack:shared': True, 'pack:foo': False, 'pack:bar': False},
                                  {'pack:shared': False, 'pack:foo': True, 'pack:bar': True},
                                  {'pack:shared': False, 'pack:foo': False, 'pack:bar': True},
                                  {'pack:shared': False, 'pack:foo': True, 'pack:bar': False},
                                  {'pack:shared': False, 'pack:foo': False, 'pack:bar': False}])
        expected = [({'arch': 'x86', 'compiler': 'gcc', 'compiler.version': '4.9',
                      'compiler.threads': 'posix', 'compiler.exception': 'dwarf2',
                      'build_type': 'Release', 'compiler.libcxx': 'libstdc++'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': True}, {},
                     {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),

                    ({'arch': 'x86', 'compiler': 'gcc', 'compiler.version': '4.9',
                      'compiler.threads': 'posix', 'compiler.exception': 'dwarf2',
                      'build_type': 'Debug', 'compiler.libcxx': 'libstdc++'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': True}, {},
                     {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),

                    ({'arch': 'x86', 'compiler': 'gcc', 'compiler.version': '4.9',
                      'compiler.threads': 'posix', 'compiler.exception': 'dwarf2',
                      'build_type': 'Release', 'compiler.libcxx': 'libstdc++'},
                     {'pack:shared': True, 'pack:foo': False, 'pack:bar': True}, {},
                     {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),

                    ({'arch': 'x86', 'compiler': 'gcc', 'compiler.version': '4.9',
                      'compiler.threads': 'posix', 'compiler.exception': 'dwarf2',
                      'build_type': 'Debug', 'compiler.libcxx': 'libstdc++'},
                     {'pack:shared': True, 'pack:foo': False, 'pack:bar': True}, {},
                     {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),

                    ({'arch': 'x86', 'compiler': 'gcc', 'compiler.version': '4.9',
                      'compiler.threads': 'posix', 'compiler.exception': 'dwarf2',
                      'build_type': 'Release', 'compiler.libcxx': 'libstdc++'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': False}, {},
                     {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),

                    ({'arch': 'x86', 'compiler': 'gcc', 'compiler.version': '4.9',
                      'compiler.threads': 'posix', 'compiler.exception': 'dwarf2',
                      'build_type': 'Debug', 'compiler.libcxx': 'libstdc++'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': False}, {},
                     {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),

                    ({'arch': 'x86', 'compiler': 'gcc', 'compiler.version': '4.9',
                      'compiler.threads': 'posix', 'compiler.exception': 'dwarf2',
                      'build_type': 'Release', 'compiler.libcxx': 'libstdc++'},
                     {'pack:shared': True, 'pack:foo': False, 'pack:bar': False}, {},
                     {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),

                    ({'arch': 'x86', 'compiler': 'gcc', 'compiler.version': '4.9',
                      'compiler.threads': 'posix', 'compiler.exception': 'dwarf2',
                      'build_type': 'Debug', 'compiler.libcxx': 'libstdc++'},
                     {'pack:shared': True, 'pack:foo': False, 'pack:bar': False}, {},
                     {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),

                    ({'arch': 'x86', 'compiler': 'gcc', 'compiler.version': '4.9',
                      'compiler.threads': 'posix', 'compiler.exception': 'dwarf2',
                      'build_type': 'Release', 'compiler.libcxx': 'libstdc++'},
                     {'pack:shared': False, 'pack:foo': True, 'pack:bar': True}, {},
                     {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),

                    ({'arch': 'x86', 'compiler': 'gcc', 'compiler.version': '4.9',
                      'compiler.threads': 'posix', 'compiler.exception': 'dwarf2',
                      'build_type': 'Debug', 'compiler.libcxx': 'libstdc++'},
                     {'pack:shared': False, 'pack:foo': True, 'pack:bar': True}, {},
                     {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),

                    ({'arch': 'x86', 'compiler': 'gcc', 'compiler.version': '4.9',
                      'compiler.threads': 'posix', 'compiler.exception': 'dwarf2',
                      'build_type': 'Release', 'compiler.libcxx': 'libstdc++'},
                     {'pack:shared': False, 'pack:foo': False, 'pack:bar': True}, {},
                     {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),

                    ({'arch': 'x86', 'compiler': 'gcc', 'compiler.version': '4.9',
                      'compiler.threads': 'posix', 'compiler.exception': 'dwarf2',
                      'build_type': 'Debug', 'compiler.libcxx': 'libstdc++'},
                     {'pack:shared': False, 'pack:foo': False, 'pack:bar': True}, {},
                     {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),

                    ({'arch': 'x86', 'compiler': 'gcc', 'compiler.version': '4.9',
                      'compiler.threads': 'posix', 'compiler.exception': 'dwarf2',
                      'build_type': 'Release', 'compiler.libcxx': 'libstdc++'},
                     {'pack:shared': False, 'pack:foo': True, 'pack:bar': False}, {},
                     {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),

                    ({'arch': 'x86', 'compiler': 'gcc', 'compiler.version': '4.9',
                      'compiler.threads': 'posix', 'compiler.exception': 'dwarf2',
                      'build_type': 'Debug', 'compiler.libcxx': 'libstdc++'},
                     {'pack:shared': False, 'pack:foo': True, 'pack:bar': False}, {},
                     {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),

                    ({'arch': 'x86', 'compiler': 'gcc', 'compiler.version': '4.9',
                      'compiler.threads': 'posix', 'compiler.exception': 'dwarf2',
                      'build_type': 'Release', 'compiler.libcxx': 'libstdc++'},
                     {'pack:shared': False, 'pack:foo': False, 'pack:bar': False}, {},
                     {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref),

                    ({'arch': 'x86', 'compiler': 'gcc', 'compiler.version': '4.9',
                      'compiler.threads': 'posix', 'compiler.exception': 'dwarf2',
                      'build_type': 'Debug', 'compiler.libcxx': 'libstdc++'},
                     {'pack:shared': False, 'pack:foo': False, 'pack:bar': False}, {},
                     {'*': [ConanFileReference.loads("mingw_installer/1.0@conan/stable")]}, ref)]

        self.assertEquals([tuple(a) for a in builds], expected)

    def test_get_osx_apple_clang_builds(self):
        ref = ConanFileReference.loads("lib/1.0@conan/stable")
        builds = get_osx_apple_clang_builds(["8.0"], ["x86_64"], "pack:shared", pure_c=False, build_types=["Debug", "Release"],
                                            cppstds=[None],
                                            options={},
                                            reference=ref)
        expected = [({'arch': 'x86_64', 'compiler.libcxx': 'libc++', 'compiler': 'apple-clang', 'compiler.version': '8.0', 'build_type': 'Debug'},
                     {'pack:shared': True}, {}, {}, ref),
                    ({'arch': 'x86_64', 'compiler.libcxx': 'libc++', 'compiler': 'apple-clang', 'compiler.version': '8.0', 'build_type': 'Release'},
                     {'pack:shared': True}, {}, {}, ref),
                    ({'arch': 'x86_64', 'compiler.libcxx': 'libc++', 'compiler': 'apple-clang', 'compiler.version': '8.0', 'build_type': 'Debug'},
                     {'pack:shared': False}, {}, {}, ref),
                    ({'arch': 'x86_64', 'compiler.libcxx': 'libc++', 'compiler': 'apple-clang', 'compiler.version': '8.0', 'build_type': 'Release'},
                     {'pack:shared': False}, {}, {}, ref)]
        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_osx_apple_clang_builds(["8.0"], ["x86_64"], "pack:shared", pure_c=True,
                                            build_types=["Debug", "Release"],
                                            cppstds=[None],
                                            options={})
        expected = [({'arch': 'x86_64', 'compiler': 'apple-clang',
                      'compiler.version': '8.0', 'build_type': 'Debug'},
                     {'pack:shared': True}, {}, {}, None),
                    ({'arch': 'x86_64', 'compiler': 'apple-clang',
                      'compiler.version': '8.0', 'build_type': 'Release'},
                     {'pack:shared': True}, {}, {}, None),
                    ({'arch': 'x86_64', 'compiler': 'apple-clang',
                      'compiler.version': '8.0', 'build_type': 'Debug'},
                     {'pack:shared': False}, {}, {}, None),
                    ({'arch': 'x86_64', 'compiler': 'apple-clang',
                      'compiler.version': '8.0', 'build_type': 'Release'},
                     {'pack:shared': False}, {}, {}, None)]
        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_osx_apple_clang_builds(["8.0"], ["x86_64"], "pack:shared", pure_c=False, build_types=["Debug"], cppstds=["14"], options={})
        expected = [({'arch': 'x86_64', 'compiler.libcxx': 'libc++', 'compiler': 'apple-clang',
                      'compiler.version': '8.0', 'build_type': 'Debug', 'compiler.cppstd': '14'},
                     {'pack:shared': True}, {}, {}, None),
                    ({'arch': 'x86_64', 'compiler.libcxx': 'libc++', 'compiler': 'apple-clang',
                      'compiler.version': '8.0', 'build_type': 'Debug', 'compiler.cppstd': '14'},
                     {'pack:shared': False}, {}, {}, None)]
        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_osx_apple_clang_builds(["8.0"], ["x86_64"], "pack:shared", pure_c=True, build_types=["Release"], cppstds=["17"], options={})
        expected = [({'arch': 'x86_64', 'compiler': 'apple-clang',
                      'compiler.version': '8.0', 'build_type': 'Release'},
                     {'pack:shared': True}, {}, {}, None),
                    ({'arch': 'x86_64', 'compiler': 'apple-clang',
                      'compiler.version': '8.0', 'build_type': 'Release'},
                     {'pack:shared': False}, {}, {}, None)]
        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_osx_apple_clang_builds(["8.0"], ["x86_64"], "pack:shared", pure_c=False,
                                            build_types=["Debug", "Release"], cppstds=[None], options={"qux:foobar": False, "foo:pkg": "bar"}, reference=ref)
        expected = [({'arch': 'x86_64', 'compiler.libcxx': 'libc++', 'compiler': 'apple-clang',
                      'compiler.version': '8.0', 'build_type': 'Debug'},
                     {'pack:shared': True, "qux:foobar": False, "foo:pkg": "bar"}, {}, {}, ref),
                    ({'arch': 'x86_64', 'compiler.libcxx': 'libc++', 'compiler': 'apple-clang',
                      'compiler.version': '8.0', 'build_type': 'Release'},
                     {'pack:shared': True, "qux:foobar": False, "foo:pkg": "bar"}, {}, {}, ref),
                    ({'arch': 'x86_64', 'compiler.libcxx': 'libc++', 'compiler': 'apple-clang',
                      'compiler.version': '8.0', 'build_type': 'Debug'},
                     {'pack:shared': False, "qux:foobar": False, "foo:pkg": "bar"}, {}, {}, ref),
                    ({'arch': 'x86_64', 'compiler.libcxx': 'libc++', 'compiler': 'apple-clang',
                      'compiler.version': '8.0', 'build_type': 'Release'},
                     {'pack:shared': False, "qux:foobar": False, "foo:pkg": "bar"}, {}, {}, ref)]
        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_osx_apple_clang_builds(["8.0"], ["x86_64"], None, pure_c=False,
                                            build_types=["Debug", "Release"],
                                            cppstds=[None],
                                            options={"qux:foobar": False, "foo:pkg": "bar"}, reference=ref)
        expected = [({'arch': 'x86_64', 'compiler.libcxx': 'libc++', 'compiler': 'apple-clang',
                      'compiler.version': '8.0', 'build_type': 'Debug'},
                     {"qux:foobar": False, "foo:pkg": "bar"}, {}, {}, ref),
                    ({'arch': 'x86_64', 'compiler.libcxx': 'libc++', 'compiler': 'apple-clang',
                      'compiler.version': '8.0', 'build_type': 'Release'},
                     {"qux:foobar": False, "foo:pkg": "bar"}, {}, {}, ref)]
        self.assertEquals([tuple(a) for a in builds], expected)

        ref = ConanFileReference.loads("lib/1.0@conan/stable")
        builds = get_osx_apple_clang_builds(["8.0"], ["x86_64"], "pack:shared", pure_c=False,
                                            build_types=["Debug", "Release"],
                                            cppstds=[None],
                                            options={},
                                            reference=ref,
                                            build_all_options_values=[
                                                {'pack:shared': True, 'pack:foo': True, 'pack:bar': True},
                                                {'pack:shared': True, 'pack:foo': False, 'pack:bar': True},
                                                {'pack:shared': True, 'pack:foo': True, 'pack:bar': False},
                                                {'pack:shared': True, 'pack:foo': False, 'pack:bar': False},
                                                {'pack:shared': False, 'pack:foo': True, 'pack:bar': True},
                                                {'pack:shared': False, 'pack:foo': False, 'pack:bar': True},
                                                {'pack:shared': False, 'pack:foo': True, 'pack:bar': False},
                                                {'pack:shared': False, 'pack:foo': False, 'pack:bar': False}])
        expected = [({'arch': 'x86_64', 'build_type': 'Debug', 'compiler': 'apple-clang',
                      'compiler.version': '8.0', 'compiler.libcxx': 'libc++'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': True}, {}, {}, ref), (
                    {'arch': 'x86_64', 'build_type': 'Release', 'compiler': 'apple-clang',
                     'compiler.version': '8.0', 'compiler.libcxx': 'libc++'},
                    {'pack:shared': True, 'pack:foo': True, 'pack:bar': True}, {}, {}, ref), (
                    {'arch': 'x86_64', 'build_type': 'Debug', 'compiler': 'apple-clang',
                     'compiler.version': '8.0', 'compiler.libcxx': 'libc++'},
                    {'pack:shared': True, 'pack:foo': False, 'pack:bar': True}, {}, {}, ref), (
                    {'arch': 'x86_64', 'build_type': 'Release', 'compiler': 'apple-clang',
                     'compiler.version': '8.0', 'compiler.libcxx': 'libc++'},
                    {'pack:shared': True, 'pack:foo': False, 'pack:bar': True}, {}, {}, ref), (
                    {'arch': 'x86_64', 'build_type': 'Debug', 'compiler': 'apple-clang',
                     'compiler.version': '8.0', 'compiler.libcxx': 'libc++'},
                    {'pack:shared': True, 'pack:foo': True, 'pack:bar': False}, {}, {}, ref), (
                    {'arch': 'x86_64', 'build_type': 'Release', 'compiler': 'apple-clang',
                     'compiler.version': '8.0', 'compiler.libcxx': 'libc++'},
                    {'pack:shared': True, 'pack:foo': True, 'pack:bar': False}, {}, {}, ref), (
                    {'arch': 'x86_64', 'build_type': 'Debug', 'compiler': 'apple-clang',
                     'compiler.version': '8.0', 'compiler.libcxx': 'libc++'},
                    {'pack:shared': True, 'pack:foo': False, 'pack:bar': False}, {}, {}, ref), (
                    {'arch': 'x86_64', 'build_type': 'Release', 'compiler': 'apple-clang',
                     'compiler.version': '8.0', 'compiler.libcxx': 'libc++'},
                    {'pack:shared': True, 'pack:foo': False, 'pack:bar': False}, {}, {}, ref), (
                    {'arch': 'x86_64', 'build_type': 'Debug', 'compiler': 'apple-clang',
                     'compiler.version': '8.0', 'compiler.libcxx': 'libc++'},
                    {'pack:shared': False, 'pack:foo': True, 'pack:bar': True}, {}, {}, ref), (
                    {'arch': 'x86_64', 'build_type': 'Release', 'compiler': 'apple-clang',
                     'compiler.version': '8.0', 'compiler.libcxx': 'libc++'},
                    {'pack:shared': False, 'pack:foo': True, 'pack:bar': True}, {}, {}, ref), (
                    {'arch': 'x86_64', 'build_type': 'Debug', 'compiler': 'apple-clang',
                     'compiler.version': '8.0', 'compiler.libcxx': 'libc++'},
                    {'pack:shared': False, 'pack:foo': False, 'pack:bar': True}, {}, {}, ref), (
                    {'arch': 'x86_64', 'build_type': 'Release', 'compiler': 'apple-clang',
                     'compiler.version': '8.0', 'compiler.libcxx': 'libc++'},
                    {'pack:shared': False, 'pack:foo': False, 'pack:bar': True}, {}, {}, ref), (
                    {'arch': 'x86_64', 'build_type': 'Debug', 'compiler': 'apple-clang',
                     'compiler.version': '8.0', 'compiler.libcxx': 'libc++'},
                    {'pack:shared': False, 'pack:foo': True, 'pack:bar': False}, {}, {}, ref), (
                    {'arch': 'x86_64', 'build_type': 'Release', 'compiler': 'apple-clang',
                     'compiler.version': '8.0', 'compiler.libcxx': 'libc++'},
                    {'pack:shared': False, 'pack:foo': True, 'pack:bar': False}, {}, {}, ref), (
                    {'arch': 'x86_64', 'build_type': 'Debug', 'compiler': 'apple-clang',
                     'compiler.version': '8.0', 'compiler.libcxx': 'libc++'},
                    {'pack:shared': False, 'pack:foo': False, 'pack:bar': False}, {}, {}, ref), (
                    {'arch': 'x86_64', 'build_type': 'Release', 'compiler': 'apple-clang',
                     'compiler.version': '8.0', 'compiler.libcxx': 'libc++'},
                    {'pack:shared': False, 'pack:foo': False, 'pack:bar': False}, {}, {}, ref)]

        self.assertEquals([tuple(a) for a in builds], expected)

    def test_get_linux_gcc_builds(self):
        builds = get_linux_gcc_builds(["6"], ["x86_64"], "pack:shared", pure_c=False, build_types=["Debug", "Release"], cppstds=[None], options={})
        expected = [({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++', 'compiler.version': '6', 'arch': 'x86_64'},
                     {'pack:shared': True}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++11', 'compiler.version': '6', 'arch': 'x86_64'},
                     {'pack:shared': True}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++', 'compiler.version': '6', 'arch': 'x86_64'},
                     {'pack:shared': True}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++11', 'compiler.version': '6', 'arch': 'x86_64'},
                     {'pack:shared': True}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++', 'compiler.version': '6', 'arch': 'x86_64'},
                     {'pack:shared': False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++11', 'compiler.version': '6', 'arch': 'x86_64'},
                     {'pack:shared': False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++', 'compiler.version': '6', 'arch': 'x86_64'},
                     {'pack:shared': False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++11', 'compiler.version': '6', 'arch': 'x86_64'},
                     {'pack:shared': False}, {}, {}, None)]
        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_linux_gcc_builds(["6"], ["x86_64"], "pack:shared", pure_c=True, build_types=["Debug", "Release"], cppstds=[None], options={})
        expected = [({'arch': 'x86_64', 'compiler.version': '6', 'build_type': 'Debug', 'compiler': 'gcc'},
                     {'pack:shared': True}, {}, {}, None),
                    ({'arch': 'x86_64', 'compiler.version': '6', 'build_type': 'Release', 'compiler': 'gcc'},
                     {'pack:shared': True}, {}, {}, None),
                    ({'arch': 'x86_64', 'compiler.version': '6', 'build_type': 'Debug', 'compiler': 'gcc'},
                     {'pack:shared': False}, {}, {}, None),
                    ({'arch': 'x86_64', 'compiler.version': '6', 'build_type': 'Release', 'compiler': 'gcc'},
                     {'pack:shared': False}, {}, {}, None)]
        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_linux_gcc_builds(["6"], ["x86_64"], "pack:shared", pure_c=False, build_types=["Debug"], cppstds=["14"], options={})
        expected = [({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6', 'arch': 'x86_64', 'compiler.cppstd': '14'},
                     {'pack:shared': True}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '6', 'arch': 'x86_64', 'compiler.cppstd': '14'},
                     {'pack:shared': True}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6', 'arch': 'x86_64', 'compiler.cppstd': '14'},
                     {'pack:shared': False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '6', 'arch': 'x86_64', 'compiler.cppstd': '14'},
                     {'pack:shared': False}, {}, {}, None)]
        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_linux_gcc_builds(["6"], ["x86_64"], "pack:shared", pure_c=True, build_types=["Debug"], cppstds=["14"], options={})
        expected = [({'arch': 'x86_64', 'compiler.version': '6', 'build_type': 'Debug', 'compiler': 'gcc'},
                     {'pack:shared': True}, {}, {}, None),
                    ({'arch': 'x86_64', 'compiler.version': '6', 'build_type': 'Debug', 'compiler': 'gcc'},
                     {'pack:shared': False}, {}, {}, None)]
        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_linux_gcc_builds(["6"], ["x86_64"], "pack:shared", pure_c=False, build_types=["Release"], cppstds=["17"], options={})
        expected = [({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6', 'arch': 'x86_64', 'compiler.cppstd': '17'},
                     {'pack:shared': True}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '6', 'arch': 'x86_64', 'compiler.cppstd': '17'},
                     {'pack:shared': True}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6', 'arch': 'x86_64', 'compiler.cppstd': '17'},
                     {'pack:shared': False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '6', 'arch': 'x86_64', 'compiler.cppstd': '17'},
                     {'pack:shared': False}, {}, {}, None)]
        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_linux_gcc_builds(["6"], ["x86_64"], "pack:shared", pure_c=True, build_types=["Release"], cppstds=["17"], options={})
        expected = [({'arch': 'x86_64', 'compiler.version': '6', 'build_type': 'Release', 'compiler': 'gcc'},
                     {'pack:shared': True}, {}, {}, None),
                    ({'arch': 'x86_64', 'compiler.version': '6', 'build_type': 'Release', 'compiler': 'gcc'},
                     {'pack:shared': False}, {}, {}, None)]
        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_linux_gcc_builds(["6"], ["x86_64"], "pack:shared", pure_c=False, build_types=["Debug", "Release"],
                                      cppstds=[None], options={"foo:bar": "qux", "pkg:qux": False})
        expected = [({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++', 'compiler.version': '6',
                      'arch': 'x86_64'},
                     {'pack:shared': True, "foo:bar": "qux", "pkg:qux": False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '6', 'arch': 'x86_64'},
                     {'pack:shared': True, "foo:bar": "qux", "pkg:qux": False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6', 'arch': 'x86_64'},
                     {'pack:shared': True, "foo:bar": "qux", "pkg:qux": False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '6', 'arch': 'x86_64'},
                     {'pack:shared': True, "foo:bar": "qux", "pkg:qux": False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++', 'compiler.version': '6',
                      'arch': 'x86_64'},
                     {'pack:shared': False, "foo:bar": "qux", "pkg:qux": False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '6', 'arch': 'x86_64'},
                     {'pack:shared': False, "foo:bar": "qux", "pkg:qux": False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6', 'arch': 'x86_64'},
                     {'pack:shared': False, "foo:bar": "qux", "pkg:qux": False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '6', 'arch': 'x86_64'},
                     {'pack:shared': False, "foo:bar": "qux", "pkg:qux": False}, {}, {}, None)]
        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_linux_gcc_builds(["6"], ["x86_64"], None, pure_c=True, build_types=["Debug", "Release"],
                                      cppstds=[None], options={"qux:bar": "foo", "*:pkg": False})
        expected = [({'arch': 'x86_64', 'compiler.version': '6', 'build_type': 'Debug', 'compiler': 'gcc'},
                     {"qux:bar": "foo", "*:pkg": False}, {}, {}, None),
                    ({'arch': 'x86_64', 'compiler.version': '6', 'build_type': 'Release', 'compiler': 'gcc'},
                     {"qux:bar": "foo", "*:pkg": False}, {}, {}, None)]
        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_linux_gcc_builds(["9"], ["x86_64"], "pack:shared", pure_c=False,
                                      build_types=["Debug", "Release"], cppstds=[None], options={},
                                      build_all_options_values=[{"pack:shared": True, "pack:foo": True, "pack:bar": True},
                                                                {"pack:shared": True, "pack:foo": False, "pack:bar": True},
                                                                {"pack:shared": True, "pack:foo": True, "pack:bar": False},
                                                                {"pack:shared": True, "pack:foo": False, "pack:bar": False},
                                                                {"pack:shared": False, "pack:foo": True, "pack:bar": True},
                                                                {"pack:shared": False, "pack:foo": False, "pack:bar": True},
                                                                {"pack:shared": False, "pack:foo": True, "pack:bar": False},
                                                                {"pack:shared": False, "pack:foo": False, "pack:bar": False}])
        expected = [({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': True}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': True}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': True}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': True}, {}, {}, None),

                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': False, 'pack:bar': True}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': False, 'pack:bar': True}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': False, 'pack:bar': True}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': False, 'pack:bar': True}, {}, {}, None),

                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': False}, {}, {}, None),

                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': False, 'pack:bar': False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': False, 'pack:bar': False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': False, 'pack:bar': False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': False, 'pack:bar': False}, {}, {}, None),

                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': True, 'pack:bar': True}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': True, 'pack:bar': True}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': True, 'pack:bar': True}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': True, 'pack:bar': True}, {}, {}, None),

                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': False, 'pack:bar': True}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': False, 'pack:bar': True}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': False, 'pack:bar': True}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': False, 'pack:bar': True}, {}, {}, None),

                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': True, 'pack:bar': False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': True, 'pack:bar': False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': True, 'pack:bar': False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': True, 'pack:bar': False}, {}, {}, None),

                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': False, 'pack:bar': False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': False, 'pack:bar': False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': False, 'pack:bar': False}, {}, {}, None),
                    ({'compiler': 'gcc', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++11',
                      'compiler.version': '9', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': False, 'pack:bar': False}, {}, {}, None)]
        self.assertEquals([tuple(a) for a in builds], expected)

    def test_get_linux_clang_builds(self):
        self.maxDiff = None
        ref = ConanFileReference.loads("lib/2.3@conan/stable")
        builds = get_linux_clang_builds(["4.0"], ["x86_64"], "pack:shared", pure_c=False,
                                        build_types=["Debug", "Release"], cppstds=[None], options={},
                                        reference=ref)
        expected = [({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++', 'compiler.version': '4.0', 'arch': 'x86_64'},
                     {'pack:shared': True}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libc++', 'compiler.version': '4.0', 'arch': 'x86_64'},
                     {'pack:shared': True}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++', 'compiler.version': '4.0', 'arch': 'x86_64'},
                     {'pack:shared': True}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libc++', 'compiler.version': '4.0', 'arch': 'x86_64'},
                     {'pack:shared': True}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++', 'compiler.version': '4.0', 'arch': 'x86_64'},
                     {'pack:shared': False}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libc++', 'compiler.version': '4.0', 'arch': 'x86_64'},
                     {'pack:shared': False}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++', 'compiler.version': '4.0', 'arch': 'x86_64'},
                     {'pack:shared': False}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libc++', 'compiler.version': '4.0', 'arch': 'x86_64'},
                     {'pack:shared': False}, {}, {}, ref)]
        b = [tuple(a) for a in builds]
        self.assertEquals(b, expected)

        builds = get_linux_clang_builds(["4.0"], ["x86_64"], "pack:shared", pure_c=True,
                                        build_types=["Debug", "Release"], cppstds=[None], options={}, reference=ref)
        expected = [({'arch': 'x86_64', 'compiler.version': '4.0', 'build_type': 'Debug', 'compiler': 'clang'},
                     {'pack:shared': True}, {}, {}, ref),
                    ({'arch': 'x86_64', 'compiler.version': '4.0', 'build_type': 'Release', 'compiler': 'clang'},
                     {'pack:shared': True}, {}, {}, ref),
                    ({'arch': 'x86_64', 'compiler.version': '4.0', 'build_type': 'Debug', 'compiler': 'clang'},
                     {'pack:shared': False}, {}, {}, ref),
                    ({'arch': 'x86_64', 'compiler.version': '4.0', 'build_type': 'Release', 'compiler': 'clang'},
                     {'pack:shared': False}, {}, {}, ref)]
        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_linux_clang_builds(["4.0"], ["x86_64"], "pack:shared", pure_c=False,
                                        build_types=["Debug"], cppstds=[None], options={}, reference=ref)
        expected = [({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '4.0', 'arch': 'x86_64'},
                     {'pack:shared': True}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libc++', 'compiler.version': '4.0',
                     'arch': 'x86_64'},
                    {'pack:shared': True}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '4.0', 'arch': 'x86_64'},
                     {'pack:shared': False}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libc++', 'compiler.version': '4.0',
                     'arch': 'x86_64'},
                    {'pack:shared': False}, {}, {}, ref)]
        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_linux_clang_builds(["4.0"], ["x86_64"], "pack:shared", pure_c=True,
                                        build_types=["Debug"], cppstds=[None], options={}, reference=ref)
        expected = [({'arch': 'x86_64', 'compiler.version': '4.0', 'build_type': 'Debug', 'compiler': 'clang'},
                     {'pack:shared': True}, {}, {}, ref),
                    ({'arch': 'x86_64', 'compiler.version': '4.0', 'build_type': 'Debug', 'compiler': 'clang'},
                     {'pack:shared': False}, {}, {}, ref)]
        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_linux_clang_builds(["4.0"], ["x86_64"], "pack:shared", pure_c=False,
                                        build_types=["Release"], cppstds=[None], options={}, reference=ref)
        expected = [({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '4.0', 'arch': 'x86_64'},
                     {'pack:shared': True}, {}, {}, ref),
                    (
                    {'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libc++', 'compiler.version': '4.0',
                     'arch': 'x86_64'},
                    {'pack:shared': True}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '4.0', 'arch': 'x86_64'},
                     {'pack:shared': False}, {}, {}, ref),
                    (
                    {'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libc++', 'compiler.version': '4.0',
                     'arch': 'x86_64'},
                    {'pack:shared': False}, {}, {}, ref)]
        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_linux_clang_builds(["4.0"], ["x86_64"], "pack:shared", pure_c=True, build_types=["Release"],
                                        cppstds=[None], options={}, reference=None)
        expected = [({'arch': 'x86_64', 'compiler.version': '4.0', 'build_type': 'Release', 'compiler': 'clang'},
                     {'pack:shared': True}, {}, {}, None),
                    ({'arch': 'x86_64', 'compiler.version': '4.0', 'build_type': 'Release', 'compiler': 'clang'},
                     {'pack:shared': False}, {}, {}, None)]
        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_linux_clang_builds(["4.0"], ["x86_64"], "pack:shared", pure_c=False,
                                        build_types=["Debug"], cppstds=[None],
                                        options={"foo:bar": "qux", "pkg:shared": True}, reference=ref)
        expected = [({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '4.0', 'arch': 'x86_64'},
                     {'pack:shared': True, "foo:bar": "qux", "pkg:shared": True}, {}, {}, ref),
                    (
                    {'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libc++', 'compiler.version': '4.0',
                     'arch': 'x86_64'},
                    {'pack:shared': True, "foo:bar": "qux", "pkg:shared": True}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '4.0', 'arch': 'x86_64'},
                     {'pack:shared': False, "foo:bar": "qux", "pkg:shared": True}, {}, {}, ref),
                    (
                    {'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libc++', 'compiler.version': '4.0',
                     'arch': 'x86_64'},
                    {'pack:shared': False, "foo:bar": "qux", "pkg:shared": True}, {}, {}, ref)]
        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_linux_clang_builds(["4.0"], ["x86_64"], None, pure_c=False,
                                        build_types=["Debug"], cppstds=[None],
                                        options={"foo:bar": "qux", "pkg:shared": True},
                                        reference=ref)
        expected = [({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '4.0', 'arch': 'x86_64'},
                     {"foo:bar": "qux", "pkg:shared": True}, {}, {}, ref),
                    (
                        {'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libc++',
                         'compiler.version': '4.0',
                         'arch': 'x86_64'},
                        {"foo:bar": "qux", "pkg:shared": True}, {}, {}, ref)]
        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_linux_clang_builds(["6.0"], ["x86_64"], "pack:shared", pure_c=False,
                                        build_types=["Debug", "Release"], cppstds=[None],
                                        options={}, reference=ref,
                                        build_all_options_values=[
                                          {"pack:shared": True, "pack:foo": True, "pack:bar": True},
                                          {"pack:shared": True, "pack:foo": False, "pack:bar": True},
                                          {"pack:shared": True, "pack:foo": True, "pack:bar": False},
                                          {"pack:shared": True, "pack:foo": False, "pack:bar": False},
                                          {"pack:shared": False, "pack:foo": True, "pack:bar": True},
                                          {"pack:shared": False, "pack:foo": False, "pack:bar": True},
                                          {"pack:shared": False, "pack:foo": True, "pack:bar": False},
                                          {"pack:shared": False, "pack:foo": False, "pack:bar": False}
                                        ])
        expected = [({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': True}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': True}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': True}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': True}, {}, {}, ref),

                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': False, 'pack:bar': True}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': False, 'pack:bar': True}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': False, 'pack:bar': True}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': False, 'pack:bar': True}, {}, {}, ref),

                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': False}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': False}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': False}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': False}, {}, {}, ref),

                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': False, 'pack:bar': False}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': False, 'pack:bar': False}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': False, 'pack:bar': False}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': True, 'pack:foo': False, 'pack:bar': False}, {}, {}, ref),

                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': True, 'pack:bar': True}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': True, 'pack:bar': True}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': True, 'pack:bar': True}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': True, 'pack:bar': True}, {}, {}, ref),

                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': False, 'pack:bar': True}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': False, 'pack:bar': True}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': False, 'pack:bar': True}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': False, 'pack:bar': True}, {}, {}, ref),

                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': True, 'pack:bar': False}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': True, 'pack:bar': False}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': True, 'pack:bar': False}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': True, 'pack:bar': False}, {}, {}, ref),

                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': False, 'pack:bar': False}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Debug', 'compiler.libcxx': 'libc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': False, 'pack:bar': False}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libstdc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': False, 'pack:bar': False}, {}, {}, ref),
                    ({'compiler': 'clang', 'build_type': 'Release', 'compiler.libcxx': 'libc++',
                      'compiler.version': '6.0', 'arch': 'x86_64'},
                     {'pack:shared': False, 'pack:foo': False, 'pack:bar': False}, {}, {}, ref)]

        b = [tuple(a) for a in builds]
        self.assertEquals(b, expected)

    def test_visual_build_generator(self):
        ref = ConanFileReference.loads("lib/1.0@conan/stable")
        builds = get_visual_builds(visual_versions=["10", "14"],
                                   archs=["x86"], visual_runtimes=["MDd", "MTd"],
                                   visual_toolsets=None,
                                   shared_option_name=None,
                                   dll_with_static_runtime=False,
                                   vs10_x86_64_enabled=True,
                                   build_types=["Debug", "Release", "RelWithDebInfo", "MinSizeRel"],
                                   cppstds=[None],
                                   options={},
                                   reference=ref)

        expected = [
        ({'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio', 'compiler.version': '10', 'compiler.runtime': 'MDd'}, {}, {}, {}, ref),
        ({'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio', 'compiler.version': '10', 'compiler.runtime': 'MTd'}, {}, {}, {}, ref),
        ({'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio', 'compiler.version': '14', 'compiler.runtime': 'MDd'}, {}, {}, {}, ref),
        ({'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio', 'compiler.version': '14', 'compiler.runtime': 'MTd'}, {}, {}, {}, ref)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_visual_builds(visual_versions=["10"], archs=["x86", "x86_64"],
                                   visual_runtimes=["MDd"], visual_toolsets=None,
                                   shared_option_name="libpng:shared",
                                   dll_with_static_runtime=True,
                                   vs10_x86_64_enabled=True,
                                   build_types=["Debug", "Release", "RelWithDebInfo", "MinSizeRel"],
                                   cppstds=[None],
                                   options={})

        expected = [
            ({'compiler.runtime': 'MDd', 'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio', 'compiler.version': '10'},
             {'libpng:shared': False}, {}, {}, None),
            ({'compiler.runtime': 'MDd', 'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio', 'compiler.version': '10'},
             {'libpng:shared': True}, {}, {}, None),
            ({'compiler.runtime': 'MDd', 'arch': 'x86_64', 'build_type': 'Debug', 'compiler': 'Visual Studio', 'compiler.version': '10'},
             {'libpng:shared': False}, {}, {}, None),
            ({'compiler.runtime': 'MDd', 'arch': 'x86_64', 'build_type': 'Debug', 'compiler': 'Visual Studio', 'compiler.version': '10'},
             {'libpng:shared': True}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_visual_builds(visual_versions=["10"], archs=["x86", "x86_64"],
                                   visual_runtimes=["MDd"],
                                   visual_toolsets=None,
                                   shared_option_name="libpng:shared",
                                   dll_with_static_runtime=True,
                                   vs10_x86_64_enabled=False,
                                   build_types=["Debug", "Release", "RelWithDebInfo", "MinSizeRel"],
                                   cppstds=['17'],
                                   options={})
        expected = [
        ({'compiler.runtime': 'MDd', 'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio', 'compiler.version': '10', 'compiler.cppstd': '17'},
          {'libpng:shared': False}, {}, {}, None),
        ({'compiler.runtime': 'MDd', 'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio', 'compiler.version': '10', 'compiler.cppstd': '17'},
          {'libpng:shared': True}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_visual_builds(visual_versions=["10"], archs=["x86", "x86_64"],
                                   visual_runtimes=["MTd"],
                                   visual_toolsets=None,
                                   shared_option_name="libpng:shared",
                                   dll_with_static_runtime=False,
                                   vs10_x86_64_enabled=False,
                                   build_types=["Debug", "Release", "RelWithDebInfo", "MinSizeRel"],
                                   cppstds=['14'],
                                   options={})
        expected = [
            ({'compiler': 'Visual Studio', 'compiler.runtime': 'MTd', 'compiler.version': '10', 'arch': 'x86', 'compiler.cppstd': '14', 'build_type': 'Debug'},
             {'libpng:shared': False}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_visual_builds(visual_versions=["10", "14"], archs=["x86"],
                                   visual_runtimes=["MDd", "MTd"],
                                   visual_toolsets=None,
                                   shared_option_name=None,
                                   dll_with_static_runtime=False,
                                   vs10_x86_64_enabled=True,
                                   build_types=["Debug"],
                                   cppstds=['14'],
                                   options={})

        expected = [
            ({'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio', 'compiler.version': '10',
              'compiler.cppstd': '14', 'compiler.runtime': 'MDd'}, {}, {}, {}, None),
            ({'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio', 'compiler.version': '10',
              'compiler.cppstd': '14', 'compiler.runtime': 'MTd'}, {}, {}, {}, None),
            ({'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio', 'compiler.version': '14',
              'compiler.cppstd': '14', 'compiler.runtime': 'MDd'}, {}, {}, {}, None),
            ({'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio', 'compiler.version': '14',
              'compiler.cppstd': '14', 'compiler.runtime': 'MTd'}, {}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_visual_builds(visual_versions=["10"], archs=["x86", "x86_64"],
                                   visual_runtimes=["MDd"],
                                   visual_toolsets=None,
                                   shared_option_name="libpng:shared",
                                   dll_with_static_runtime=True,
                                   vs10_x86_64_enabled=True,
                                   build_types=["Debug"],
                                   cppstds=[None],
                                   options={})

        expected = [
            ({'compiler.runtime': 'MDd', 'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio',
              'compiler.version': '10'},
             {'libpng:shared': False}, {}, {}, None),
            ({'compiler.runtime': 'MDd', 'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio',
              'compiler.version': '10'},
             {'libpng:shared': True}, {}, {}, None),
            ({'compiler.runtime': 'MDd', 'arch': 'x86_64', 'build_type': 'Debug', 'compiler': 'Visual Studio',
              'compiler.version': '10'},
             {'libpng:shared': False}, {}, {}, None),
            ({'compiler.runtime': 'MDd', 'arch': 'x86_64', 'build_type': 'Debug', 'compiler': 'Visual Studio',
              'compiler.version': '10'},
             {'libpng:shared': True}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_visual_builds(visual_versions=["10"], archs=["x86", "x86_64"],
                                   visual_runtimes=["MDd"],
                                   visual_toolsets=None,
                                   shared_option_name="libpng:shared",
                                   dll_with_static_runtime=True,
                                   vs10_x86_64_enabled=False,
                                   build_types=["Debug"],
                                   cppstds=['20'],
                                   options={})
        expected = [
            ({'compiler.runtime': 'MDd', 'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio',
              'compiler.cppstd': '20', 'compiler.version': '10'},
             {'libpng:shared': False}, {}, {}, None),
            ({'compiler.runtime': 'MDd', 'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio',
              'compiler.cppstd': '20', 'compiler.version': '10'},
             {'libpng:shared': True}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_visual_builds(visual_versions=["10"], archs=["x86", "x86_64"],
                                   visual_runtimes=["MTd"],
                                   visual_toolsets=None,
                                   shared_option_name="libpng:shared",
                                   dll_with_static_runtime=False,
                                   vs10_x86_64_enabled=False,
                                   build_types=["Debug"],
                                   cppstds=['20'],
                                   options={})
        expected = [
            ({'compiler': 'Visual Studio', 'compiler.runtime': 'MTd', 'compiler.version': '10', 'arch': 'x86',
              'compiler.cppstd': '20', 'build_type': 'Debug'},
             {'libpng:shared': False}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        #############

        builds = get_visual_builds(visual_versions=["10", "14"], archs=["x86_64"],
                                   visual_runtimes=["MD", "MT"],
                                   visual_toolsets=None,
                                   shared_option_name=None,
                                   dll_with_static_runtime=False,
                                   vs10_x86_64_enabled=True,
                                   build_types=["Release"],
                                   cppstds=['14'],
                                   options={})

        expected = [
            ({'arch': 'x86_64', 'build_type': 'Release', 'compiler': 'Visual Studio', 'compiler.version': '10',
              'compiler.cppstd': '14', 'compiler.runtime': 'MD'}, {}, {}, {}, None),
            ({'arch': 'x86_64', 'build_type': 'Release', 'compiler': 'Visual Studio', 'compiler.version': '10',
              'compiler.cppstd': '14', 'compiler.runtime': 'MT'}, {}, {}, {}, None),
            ({'arch': 'x86_64', 'build_type': 'Release', 'compiler': 'Visual Studio', 'compiler.version': '14',
              'compiler.cppstd': '14', 'compiler.runtime': 'MD'}, {}, {}, {}, None),
            ({'arch': 'x86_64', 'build_type': 'Release', 'compiler': 'Visual Studio', 'compiler.version': '14',
              'compiler.cppstd': '14', 'compiler.runtime': 'MT'}, {}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_visual_builds(visual_versions=["10"], archs=["x86", "x86_64"],
                                   visual_runtimes=["MD"],
                                   visual_toolsets=None,
                                   shared_option_name="libpng:shared",
                                   dll_with_static_runtime=True,
                                   vs10_x86_64_enabled=True,
                                   build_types=["Release"],
                                   cppstds=[None],
                                   options={})

        expected = [
            ({'compiler.runtime': 'MD', 'arch': 'x86', 'build_type': 'Release', 'compiler': 'Visual Studio',
              'compiler.version': '10'},
             {'libpng:shared': False}, {}, {}, None),
            ({'compiler.runtime': 'MD', 'arch': 'x86', 'build_type': 'Release', 'compiler': 'Visual Studio',
              'compiler.version': '10'},
             {'libpng:shared': True}, {}, {}, None),
            ({'compiler.runtime': 'MD', 'arch': 'x86_64', 'build_type': 'Release', 'compiler': 'Visual Studio',
              'compiler.version': '10'},
             {'libpng:shared': False}, {}, {}, None),
            ({'compiler.runtime': 'MD', 'arch': 'x86_64', 'build_type': 'Release', 'compiler': 'Visual Studio',
              'compiler.version': '10'},
             {'libpng:shared': True}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_visual_builds(visual_versions=["10"], archs=["x86", "x86_64"],
                                   visual_runtimes=["MD"],
                                   visual_toolsets=None,
                                   shared_option_name="libpng:shared",
                                   dll_with_static_runtime=True,
                                   vs10_x86_64_enabled=False,
                                   build_types=["Release"],
                                   cppstds=['17'],
                                   options={})
        expected = [
            ({'compiler.runtime': 'MD', 'arch': 'x86', 'build_type': 'Release', 'compiler': 'Visual Studio',
              'compiler.cppstd': '17', 'compiler.version': '10'},
             {'libpng:shared': False}, {}, {}, None),
            ({'compiler.runtime': 'MD', 'arch': 'x86', 'build_type': 'Release', 'compiler': 'Visual Studio',
              'compiler.cppstd': '17', 'compiler.version': '10'},
             {'libpng:shared': True}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_visual_builds(visual_versions=["10"], archs=["x86", "x86_64"],
                                   visual_runtimes=["MT"],
                                   visual_toolsets=None,
                                   shared_option_name="libpng:shared",
                                   dll_with_static_runtime=False,
                                   vs10_x86_64_enabled=False,
                                   build_types=["Release"],
                                   cppstds=['17'],
                                   options={})
        expected = [
            ({'compiler': 'Visual Studio', 'compiler.runtime': 'MT',
              'compiler.cppstd': '17', 'compiler.version': '10', 'arch': 'x86',
              'build_type': 'Release'},
             {'libpng:shared': False}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        #############

        builds = get_visual_builds(visual_versions=["10", "14"], archs=["x86_64"],
                                   visual_runtimes=["MD", "MT"],
                                   visual_toolsets=None,
                                   shared_option_name=None,
                                   dll_with_static_runtime=False,
                                   vs10_x86_64_enabled=True,
                                   build_types=["RelWithDebInfo"],
                                   cppstds=[None],
                                   options={})

        expected = [
            ({'arch': 'x86_64', 'build_type': 'RelWithDebInfo', 'compiler': 'Visual Studio', 'compiler.version': '10',
              'compiler.runtime': 'MD'}, {}, {}, {}, None),
            ({'arch': 'x86_64', 'build_type': 'RelWithDebInfo', 'compiler': 'Visual Studio', 'compiler.version': '10',
              'compiler.runtime': 'MT'}, {}, {}, {}, None),
            ({'arch': 'x86_64', 'build_type': 'RelWithDebInfo', 'compiler': 'Visual Studio', 'compiler.version': '14',
              'compiler.runtime': 'MD'}, {}, {}, {}, None),
            ({'arch': 'x86_64', 'build_type': 'RelWithDebInfo', 'compiler': 'Visual Studio', 'compiler.version': '14',
              'compiler.runtime': 'MT'}, {}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_visual_builds(visual_versions=["10"], archs=["x86", "x86_64"],
                                   visual_runtimes=["MD"],
                                   visual_toolsets=None,
                                   shared_option_name="libpng:shared",
                                   dll_with_static_runtime=True,
                                   vs10_x86_64_enabled=True,
                                   build_types=["RelWithDebInfo"],
                                   cppstds=['14'],
                                   options={})

        expected = [
            ({'compiler.runtime': 'MD', 'arch': 'x86', 'build_type': 'RelWithDebInfo', 'compiler': 'Visual Studio',
              'compiler.cppstd': '14', 'compiler.version': '10'},
             {'libpng:shared': False}, {}, {}, None),
            ({'compiler.runtime': 'MD', 'arch': 'x86', 'build_type': 'RelWithDebInfo', 'compiler': 'Visual Studio',
              'compiler.cppstd': '14', 'compiler.version': '10'},
             {'libpng:shared': True}, {}, {}, None),
            ({'compiler.runtime': 'MD', 'arch': 'x86_64', 'build_type': 'RelWithDebInfo', 'compiler': 'Visual Studio',
              'compiler.cppstd': '14', 'compiler.version': '10'},
             {'libpng:shared': False}, {}, {}, None),
            ({'compiler.runtime': 'MD', 'arch': 'x86_64', 'build_type': 'RelWithDebInfo', 'compiler': 'Visual Studio',
              'compiler.cppstd': '14', 'compiler.version': '10'},
             {'libpng:shared': True}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_visual_builds(visual_versions=["10"], archs=["x86", "x86_64"],
                                   visual_runtimes=["MD"],
                                   visual_toolsets=None,
                                   shared_option_name="libpng:shared",
                                   dll_with_static_runtime=True,
                                   vs10_x86_64_enabled=False,
                                   build_types=["RelWithDebInfo"],
                                   cppstds=['14'],
                                   options={})
        expected = [
            ({'compiler.runtime': 'MD', 'arch': 'x86', 'build_type': 'RelWithDebInfo', 'compiler': 'Visual Studio',
              'compiler.cppstd': '14', 'compiler.version': '10'},
             {'libpng:shared': False}, {}, {}, None),
            ({'compiler.runtime': 'MD', 'arch': 'x86', 'build_type': 'RelWithDebInfo', 'compiler': 'Visual Studio',
              'compiler.cppstd': '14', 'compiler.version': '10'},
             {'libpng:shared': True}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_visual_builds(visual_versions=["10"], archs=["x86", "x86_64"],
                                   visual_runtimes=["MT"], visual_toolsets=None,
                                   shared_option_name="libpng:shared",
                                   dll_with_static_runtime=False,
                                   vs10_x86_64_enabled=False,
                                   build_types=["RelWithDebInfo"],
                                   cppstds=[None],
                                   options={})
        expected = [
            ({'compiler': 'Visual Studio', 'compiler.runtime': 'MT', 'compiler.version': '10', 'arch': 'x86',
              'build_type': 'RelWithDebInfo'},
             {'libpng:shared': False}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        #############

        builds = get_visual_builds(visual_versions=["10", "14"], archs=["x86_64"],
                                   visual_runtimes=["MD", "MT"], visual_toolsets=None,
                                   shared_option_name=None,
                                   dll_with_static_runtime=False,
                                   vs10_x86_64_enabled=True,
                                   build_types=["MinSizeRel"],
                                   cppstds=['20'],
                                   options={})

        expected = [
            ({'arch': 'x86_64', 'build_type': 'MinSizeRel', 'compiler': 'Visual Studio', 'compiler.version': '10',
              'compiler.cppstd': '20', 'compiler.runtime': 'MD'}, {}, {}, {}, None),
            ({'arch': 'x86_64', 'build_type': 'MinSizeRel', 'compiler': 'Visual Studio', 'compiler.version': '10',
              'compiler.cppstd': '20', 'compiler.runtime': 'MT'}, {}, {}, {}, None),
            ({'arch': 'x86_64', 'build_type': 'MinSizeRel', 'compiler': 'Visual Studio', 'compiler.version': '14',
              'compiler.cppstd': '20', 'compiler.runtime': 'MD'}, {}, {}, {}, None),
            ({'arch': 'x86_64', 'build_type': 'MinSizeRel', 'compiler': 'Visual Studio', 'compiler.version': '14',
              'compiler.cppstd': '20', 'compiler.runtime': 'MT'}, {}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_visual_builds(visual_versions=["10"], archs=["x86", "x86_64"],
                                   visual_runtimes=["MD"], visual_toolsets=None,
                                   shared_option_name="libpng:shared",
                                   dll_with_static_runtime=True,
                                   vs10_x86_64_enabled=True,
                                   build_types=["MinSizeRel"],
                                   cppstds=[None],
                                   options={})

        expected = [
            ({'compiler.runtime': 'MD', 'arch': 'x86', 'build_type': 'MinSizeRel', 'compiler': 'Visual Studio',
              'compiler.version': '10'},
             {'libpng:shared': False}, {}, {}, None),
            ({'compiler.runtime': 'MD', 'arch': 'x86', 'build_type': 'MinSizeRel', 'compiler': 'Visual Studio',
              'compiler.version': '10'},
             {'libpng:shared': True}, {}, {}, None),
            ({'compiler.runtime': 'MD', 'arch': 'x86_64', 'build_type': 'MinSizeRel', 'compiler': 'Visual Studio',
              'compiler.version': '10'},
             {'libpng:shared': False}, {}, {}, None),
            ({'compiler.runtime': 'MD', 'arch': 'x86_64', 'build_type': 'MinSizeRel', 'compiler': 'Visual Studio',
              'compiler.version': '10'},
             {'libpng:shared': True}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_visual_builds(visual_versions=["10"], archs=["x86", "x86_64"],
                                   visual_runtimes=["MD"], visual_toolsets=None,
                                   shared_option_name="libpng:shared",
                                   dll_with_static_runtime=True,
                                   vs10_x86_64_enabled=False,
                                   build_types=["MinSizeRel"],
                                   cppstds=['14'],
                                   options={})
        expected = [
            ({'compiler.runtime': 'MD', 'arch': 'x86', 'build_type': 'MinSizeRel', 'compiler': 'Visual Studio',
              'compiler.cppstd': '14', 'compiler.version': '10'},
             {'libpng:shared': False}, {}, {}, None),
            ({'compiler.runtime': 'MD', 'arch': 'x86', 'build_type': 'MinSizeRel', 'compiler': 'Visual Studio',
              'compiler.cppstd': '14', 'compiler.version': '10'},
             {'libpng:shared': True}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_visual_builds(visual_versions=["10"], archs=["x86", "x86_64"],
                                   visual_runtimes=["MT"], visual_toolsets=None,
                                   shared_option_name="libpng:shared",
                                   dll_with_static_runtime=False,
                                   vs10_x86_64_enabled=False,
                                   build_types=["MinSizeRel"],
                                   cppstds=[None],
                                   options={})
        expected = [
            ({'compiler': 'Visual Studio', 'compiler.runtime': 'MT', 'compiler.version': '10', 'arch': 'x86',
              'build_type': 'MinSizeRel'},
             {'libpng:shared': False}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        #############

        builds = get_visual_builds(visual_versions=["10", "14"], archs=["x86"],
                                   visual_runtimes=["MD", "MDd"], visual_toolsets=None,
                                   shared_option_name=None,
                                   dll_with_static_runtime=False,
                                   vs10_x86_64_enabled=True,
                                   build_types=["Debug", "Release", "RelWithDebInfo", "MinSizeRel"],
                                   cppstds=['14'],
                                   options={},
                                   reference=ref)

        expected = [
        ({'arch': 'x86', 'build_type': 'MinSizeRel', 'compiler': 'Visual Studio', 'compiler.version': '10', 'compiler.cppstd': '14', 'compiler.runtime': 'MD'}, {}, {}, {}, ref),
        ({'arch': 'x86', 'build_type': 'RelWithDebInfo', 'compiler': 'Visual Studio', 'compiler.version': '10', 'compiler.cppstd': '14', 'compiler.runtime': 'MD'}, {}, {}, {}, ref),
        ({'arch': 'x86', 'build_type': 'Release', 'compiler': 'Visual Studio', 'compiler.version': '10', 'compiler.cppstd': '14', 'compiler.runtime': 'MD'}, {}, {}, {}, ref),
        ({'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio', 'compiler.version': '10', 'compiler.cppstd': '14', 'compiler.runtime': 'MDd'}, {}, {}, {}, ref),
        ({'arch': 'x86', 'build_type': 'MinSizeRel', 'compiler': 'Visual Studio', 'compiler.version': '14', 'compiler.cppstd': '14', 'compiler.runtime': 'MD'}, {}, {}, {}, ref),
        ({'arch': 'x86', 'build_type': 'RelWithDebInfo', 'compiler': 'Visual Studio', 'compiler.version': '14', 'compiler.cppstd': '14', 'compiler.runtime': 'MD'}, {}, {}, {}, ref),
        ({'arch': 'x86', 'build_type': 'Release', 'compiler': 'Visual Studio', 'compiler.version': '14', 'compiler.cppstd': '14', 'compiler.runtime': 'MD'}, {}, {}, {}, ref),
        ({'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio', 'compiler.version': '14', 'compiler.cppstd': '14', 'compiler.runtime': 'MDd'}, {}, {}, {}, ref)]

        self.assertEquals([tuple(a) for a in builds], expected)

        #############

        builds = get_visual_builds(visual_versions=["10", "14"], archs=["x86"],
                                   visual_runtimes=["MDd", "MTd"], visual_toolsets=None,
                                   shared_option_name=None,
                                   dll_with_static_runtime=False,
                                   vs10_x86_64_enabled=True,
                                   build_types=["Debug", "Release", "RelWithDebInfo", "MinSizeRel"],
                                   cppstds=[None],
                                   options={"msvc:sdk": 10, "pkg:shared": True},
                                   reference=ref)

        expected = [
            ({'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio', 'compiler.version': '10',
              'compiler.runtime': 'MDd'}, {"msvc:sdk": 10, "pkg:shared": True}, {}, {}, ref),
            ({'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio', 'compiler.version': '10',
              'compiler.runtime': 'MTd'}, {"msvc:sdk": 10, "pkg:shared": True}, {}, {}, ref),
            ({'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio', 'compiler.version': '14',
              'compiler.runtime': 'MDd'}, {"msvc:sdk": 10, "pkg:shared": True}, {}, {}, ref),
            ({'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio', 'compiler.version': '14',
              'compiler.runtime': 'MTd'}, {"msvc:sdk": 10, "pkg:shared": True}, {}, {}, ref)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_visual_builds(visual_versions=["10"], archs=["x86", "x86_64"],
                                   visual_runtimes=["MDd"], visual_toolsets=None,
                                   shared_option_name="libpng:shared",
                                   dll_with_static_runtime=True,
                                   vs10_x86_64_enabled=True,
                                   build_types=["Debug", "Release", "RelWithDebInfo", "MinSizeRel"],
                                   cppstds=['17'],
                                   options={"pkg:shared": False, "pkg:foo": "bar"})

        expected = [
            ({'compiler.runtime': 'MDd', 'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio',
              'compiler.version': '10', 'compiler.cppstd': '17'},
             {'libpng:shared': False, "pkg:shared": False, "pkg:foo": "bar"}, {}, {}, None),
            ({'compiler.runtime': 'MDd', 'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio',
              'compiler.version': '10', 'compiler.cppstd': '17'},
             {'libpng:shared': True, "pkg:shared": False, "pkg:foo": "bar"}, {}, {}, None),
            ({'compiler.runtime': 'MDd', 'arch': 'x86_64', 'build_type': 'Debug', 'compiler': 'Visual Studio',
              'compiler.version': '10', 'compiler.cppstd': '17'},
             {'libpng:shared': False, "pkg:shared": False, "pkg:foo": "bar"}, {}, {}, None),
            ({'compiler.runtime': 'MDd', 'arch': 'x86_64', 'build_type': 'Debug', 'compiler': 'Visual Studio',
              'compiler.version': '10', 'compiler.cppstd': '17'},
             {'libpng:shared': True, "pkg:shared": False, "pkg:foo": "bar"}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        builds = get_visual_builds(visual_versions=["10"], archs=["x86", "x86_64"],
                                   visual_runtimes=["MDd"], visual_toolsets=None,
                                   shared_option_name="libpng:shared",
                                   dll_with_static_runtime=True,
                                   vs10_x86_64_enabled=False,
                                   build_types=["Debug", "Release", "RelWithDebInfo", "MinSizeRel"],
                                   cppstds=['14'],
                                   options={"pkg:shared": False, "pkg:fPIC": False})
        expected = [
            ({'compiler.runtime': 'MDd', 'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio',
              'compiler.version': '10', 'compiler.cppstd': '14'},
             {'libpng:shared': False, "pkg:shared": False, "pkg:fPIC": False}, {}, {}, None),
            ({'compiler.runtime': 'MDd', 'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio',
              'compiler.version': '10', 'compiler.cppstd': '14'},
             {'libpng:shared': True, "pkg:shared": False, "pkg:fPIC": False}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

    def test_visual_toolsets(self):

        builds = get_visual_builds(visual_versions=["15"], archs=["x86"],
                                   visual_runtimes=["MDd"], visual_toolsets={"15": ["v140",
                                                                                    "v140_xp"]},
                                   shared_option_name=None,
                                   dll_with_static_runtime=True,
                                   vs10_x86_64_enabled=False,
                                   build_types=["Debug", "Release", "RelWithDebInfo", "MinSizeRel"],
                                   cppstds=['17'],
                                   options={})
        expected = [
            ({'compiler.runtime': 'MDd', 'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio',
              'compiler.version': '15', 'compiler.cppstd': '17', 'compiler.toolset': 'v140'},
             {}, {}, {}, None),
            ({'compiler.runtime': 'MDd', 'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio',
              'compiler.version': '15', 'compiler.cppstd': '17', 'compiler.toolset': 'v140_xp'},
             {}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        # Same with environment passing None in the parameter
        with tools.environment_append({"CONAN_VISUAL_TOOLSETS": "15=v140;v140_xp,11=v140;v140_xp"}):
            builds = get_visual_builds(visual_versions=["15"], archs=["x86"],
                                       visual_runtimes=["MDd"], visual_toolsets=None,
                                       shared_option_name=None,
                                       dll_with_static_runtime=True,
                                       vs10_x86_64_enabled=False,
                                       build_types=["Debug", "Release", "RelWithDebInfo", "MinSizeRel"],
                                       cppstds=['17'],
                                       options={})
            self.assertEquals([tuple(a) for a in builds], expected)

        # Invalid mapping generates builds without toolsets (visual 10 != visual 15)
        builds = get_visual_builds(visual_versions=["15"], archs=["x86"],
                                   visual_runtimes=["MDd"], visual_toolsets={"10": ["v140",
                                                                                    "v140_xp"]},
                                   shared_option_name=None,
                                   dll_with_static_runtime=True,
                                   vs10_x86_64_enabled=False,
                                   build_types=["Debug", "Release", "RelWithDebInfo", "MinSizeRel"],
                                   cppstds=['14', '17'],
                                   options={})
        expected = [
            ({'compiler.runtime': 'MDd', 'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio',
              'compiler.cppstd': '14', 'compiler.version': '15'},
             {}, {}, {}, None),
            ({'compiler.runtime': 'MDd', 'arch': 'x86', 'build_type': 'Debug', 'compiler': 'Visual Studio',
              'compiler.cppstd': '17', 'compiler.version': '15'},
             {}, {}, {}, None)]

        self.assertEquals([tuple(a) for a in builds], expected)

        ref = ConanFileReference.loads("lib/1.0@conan/stable")
        builds = get_visual_builds(visual_versions=["10", "14"],
                                   archs=["x86"], visual_runtimes=["MDd", "MTd"],
                                   visual_toolsets=None,
                                   shared_option_name=None,
                                   dll_with_static_runtime=False,
                                   vs10_x86_64_enabled=True,
                                   build_types=["Debug", "Release", "RelWithDebInfo", "MinSizeRel"],
                                   cppstds=[None],
                                   options={},
                                   reference=ref,
                                   build_all_options_values=[
                                       {'pack:shared': True, 'pack:foo': True, 'pack:bar': True},
                                       {'pack:shared': True, 'pack:foo': False, 'pack:bar': True},
                                       {'pack:shared': True, 'pack:foo': True, 'pack:bar': False},
                                       {'pack:shared': True, 'pack:foo': False, 'pack:bar': False},
                                       {'pack:shared': False, 'pack:foo': True, 'pack:bar': True},
                                       {'pack:shared': False, 'pack:foo': False, 'pack:bar': True},
                                       {'pack:shared': False, 'pack:foo': True, 'pack:bar': False},
                                       {'pack:shared': False, 'pack:foo': False, 'pack:bar': False}]
                                   )

        expected = [({'compiler': 'Visual Studio', 'compiler.version': '10', 'arch': 'x86',
                      'build_type': 'Debug', 'compiler.runtime': 'MDd'},
                     {'pack:shared': True, 'pack:foo': True, 'pack:bar': True}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '10', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MDd'},
                    {'pack:shared': True, 'pack:foo': False, 'pack:bar': True}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '10', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MDd'},
                    {'pack:shared': True, 'pack:foo': True, 'pack:bar': False}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '10', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MDd'},
                    {'pack:shared': True, 'pack:foo': False, 'pack:bar': False}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '10', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MDd'},
                    {'pack:shared': False, 'pack:foo': True, 'pack:bar': True}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '10', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MDd'},
                    {'pack:shared': False, 'pack:foo': False, 'pack:bar': True}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '10', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MDd'},
                    {'pack:shared': False, 'pack:foo': True, 'pack:bar': False}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '10', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MDd'},
                    {'pack:shared': False, 'pack:foo': False, 'pack:bar': False}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '10', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MTd'},
                    {'pack:shared': True, 'pack:foo': True, 'pack:bar': True}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '10', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MTd'},
                    {'pack:shared': True, 'pack:foo': False, 'pack:bar': True}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '10', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MTd'},
                    {'pack:shared': True, 'pack:foo': True, 'pack:bar': False}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '10', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MTd'},
                    {'pack:shared': True, 'pack:foo': False, 'pack:bar': False}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '10', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MTd'},
                    {'pack:shared': False, 'pack:foo': True, 'pack:bar': True}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '10', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MTd'},
                    {'pack:shared': False, 'pack:foo': False, 'pack:bar': True}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '10', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MTd'},
                    {'pack:shared': False, 'pack:foo': True, 'pack:bar': False}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '10', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MTd'},
                    {'pack:shared': False, 'pack:foo': False, 'pack:bar': False}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '14', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MDd'},
                    {'pack:shared': True, 'pack:foo': True, 'pack:bar': True}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '14', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MDd'},
                    {'pack:shared': True, 'pack:foo': False, 'pack:bar': True}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '14', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MDd'},
                    {'pack:shared': True, 'pack:foo': True, 'pack:bar': False}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '14', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MDd'},
                    {'pack:shared': True, 'pack:foo': False, 'pack:bar': False}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '14', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MDd'},
                    {'pack:shared': False, 'pack:foo': True, 'pack:bar': True}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '14', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MDd'},
                    {'pack:shared': False, 'pack:foo': False, 'pack:bar': True}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '14', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MDd'},
                    {'pack:shared': False, 'pack:foo': True, 'pack:bar': False}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '14', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MDd'},
                    {'pack:shared': False, 'pack:foo': False, 'pack:bar': False}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '14', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MTd'},
                    {'pack:shared': True, 'pack:foo': True, 'pack:bar': True}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '14', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MTd'},
                    {'pack:shared': True, 'pack:foo': False, 'pack:bar': True}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '14', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MTd'},
                    {'pack:shared': True, 'pack:foo': True, 'pack:bar': False}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '14', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MTd'},
                    {'pack:shared': True, 'pack:foo': False, 'pack:bar': False}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '14', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MTd'},
                    {'pack:shared': False, 'pack:foo': True, 'pack:bar': True}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '14', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MTd'},
                    {'pack:shared': False, 'pack:foo': False, 'pack:bar': True}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '14', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MTd'},
                    {'pack:shared': False, 'pack:foo': True, 'pack:bar': False}, {}, {}, ref), (
                    {'compiler': 'Visual Studio', 'compiler.version': '14', 'arch': 'x86',
                     'build_type': 'Debug', 'compiler.runtime': 'MTd'},
                    {'pack:shared': False, 'pack:foo': False, 'pack:bar': False}, {}, {}, ref)]

        self.assertEquals([tuple(a) for a in builds], expected)


from conans.model.version import Version

from cpt import get_client_version


class Uploader(object):

    def __init__(self, conan_api, remote_manager, auth_manager, printer, upload_retry):
        self.conan_api = conan_api
        self.remote_manager = remote_manager
        self.auth_manager = auth_manager
        self.printer = printer
        self._upload_retry = upload_retry
        if not self._upload_retry:
            self._upload_retry = 0

    def upload_recipe(self, reference, upload):
        self._upload_artifacts(reference, upload)

    def upload_packages(self, reference, upload, package_id):
        self._upload_artifacts(reference, upload, package_id)

    def _upload_artifacts(self, reference, upload, package_id=None):
        client_version = get_client_version()
        remote_name = self.remote_manager.upload_remote_name
        if not remote_name:
            self.printer.print_message("Upload skipped, not upload remote available")
            return
        if not self.auth_manager.credentials_ready(remote_name):
            self.printer.print_message("Upload skipped, credentials for remote '%s' not available" % remote_name)
            return

        if upload:

            self.printer.print_message("Uploading packages for '%s'" % str(reference))
            self.auth_manager.login(remote_name)

            if client_version < Version("1.7.0"):
                self.conan_api.upload(str(reference),
                                      package=package_id,
                                      remote=remote_name,
                                      force=True,
                                      retry=int(self._upload_retry))
            elif client_version < Version("1.8.0"):
                self.conan_api.upload(str(reference),
                                      package=package_id,
                                      remote_name=remote_name,
                                      force=True,
                                      retry=int(self._upload_retry))
            else:
                all_packages = package_id != None
                self.conan_api.upload(str(reference),
                                      all_packages=all_packages,
                                      remote_name=remote_name,
                                      retry=int(self._upload_retry))



import os
import sys
import subprocess
from collections import namedtuple

from conans import tools
from conans.model.version import Version
from conans.model.ref import ConanFileReference

from cpt import __version__ as package_tools_version, get_client_version
from cpt.config import ConfigManager
from cpt.printer import Printer
from cpt.profiles import load_profile, patch_default_base_profile


class CreateRunner(object):

    def __init__(self, profile_abs_path, reference, conan_api, uploader,
                 exclude_vcvars_precommand=False, build_policy=None, runner=None,
                 cwd=None, printer=None, upload=False, upload_only_recipe=None,
                 test_folder=None, config_url=None, config_args=None,
                 upload_dependencies=None, conanfile=None, skip_recipe_export=False,
                 update_dependencies=False, lockfile=None):

        self.printer = printer or Printer()
        self._cwd = cwd or os.getcwd()
        self._uploader = uploader
        self._upload = upload
        self._conan_api = conan_api
        self._profile_abs_path = profile_abs_path
        self._reference = reference
        self._exclude_vcvars_precommand = exclude_vcvars_precommand
        self._build_policy =  build_policy.split(",") if \
                              isinstance(build_policy, str) else \
                              build_policy
        self._build_policy = self._build_policy or []
        self._runner = PrintRunner(runner or os.system, self.printer)
        self._test_folder = test_folder
        self._config_url = config_url
        self._config_args = config_args
        self._upload_only_recipe = upload_only_recipe
        self._conanfile = conanfile
        self._lockfile = lockfile
        self._upload_dependencies = upload_dependencies.split(",") if \
                                    isinstance(upload_dependencies, str) else \
                                    upload_dependencies
        self._upload_dependencies = self._upload_dependencies or []
        self.skip_recipe_export = skip_recipe_export
        self._update_dependencies = update_dependencies
        self._results = None

        patch_default_base_profile(conan_api, profile_abs_path)
        client_version = get_client_version()

        if client_version < Version("1.12.0"):
            cache = self._conan_api._client_cache
        elif client_version < Version("1.18.0"):
            cache = self._conan_api._cache
        else:
            if not conan_api.app:
                conan_api.create_app()
            cache = conan_api.app.cache

        self._profile = load_profile(profile_abs_path, cache)

    @property
    def settings(self):
        return self._profile.settings

    @property
    def results(self):
        return self._results

    def run(self):
        client_version = get_client_version()

        if self._config_url:
            ConfigManager(self._conan_api, self.printer).install(url=self._config_url, args=self._config_args)

        context = tools.no_op()
        compiler = self.settings.get("compiler", None)
        if not self._exclude_vcvars_precommand:
            if compiler == "Visual Studio" and "compiler.version" in self.settings:
                compiler_set = namedtuple("compiler", "version")(self.settings["compiler.version"])
                mock_sets = namedtuple("mock_settings",
                                       "arch compiler get_safe")(self.settings["arch"], compiler_set,
                                                                 lambda x: self.settings.get(x, None))
                context = tools.vcvars(mock_sets)
        with context:
            self.printer.print_rule()
            self.printer.print_profile(tools.load(self._profile_abs_path))

            with self.printer.foldable_output("conan_create"):
                if client_version < Version("1.10.0"):
                    name, version, user, channel = self._reference
                else:
                    name, version, user, channel, _ = self._reference

                if self._build_policy:
                    self._build_policy = [] if self._build_policy == ["all"] else self._build_policy
                # https://github.com/conan-io/conan-package-tools/issues/184
                with tools.environment_append({"_CONAN_CREATE_COMMAND_": "1"}):
                    params = {"name": name, "version": version, "user": user,
                              "channel": channel, "build_modes": self._build_policy,
                              "profile_name": self._profile_abs_path}
                    self.printer.print_message("Calling 'conan create'")
                    self.printer.print_dict(params)
                    with tools.chdir(self._cwd):
                        if Version(client_version) >= "1.8.0":
                            from conans.errors import ConanInvalidConfiguration
                            exc_class = ConanInvalidConfiguration
                        else:
                            exc_class = None

                        try:
                            if client_version < Version("1.12.0"):
                                self._results = self._conan_api.create(self._conanfile, name=name, version=version,
                                                        user=user, channel=channel,
                                                        build_modes=self._build_policy,
                                                        profile_name=self._profile_abs_path,
                                                        test_folder=self._test_folder,
                                                        not_export=self.skip_recipe_export,
                                                        update=self._update_dependencies)
                            else:
                                self._results = self._conan_api.create(self._conanfile, name=name, version=version,
                                                        user=user, channel=channel,
                                                        build_modes=self._build_policy,
                                                        profile_names=[self._profile_abs_path],
                                                        test_folder=self._test_folder,
                                                        not_export=self.skip_recipe_export,
                                                        update=self._update_dependencies,
                                                        lockfile=self._lockfile)
                        except exc_class as e:
                            self.printer.print_rule()
                            self.printer.print_message("Skipped configuration by the recipe: "
                                                       "%s" % str(e))
                            self.printer.print_rule()
                            return
                        for installed in self._results['installed']:
                            reference = installed["recipe"]["id"]
                            if client_version >= Version("1.10.0"):
                                reference = ConanFileReference.loads(reference)
                                reference = str(reference.copy_clear_rev())
                            if ((reference == str(self._reference)) or \
                               (reference in self._upload_dependencies) or \
                               ("all" in self._upload_dependencies)) and \
                               installed['packages']:
                                package_id = installed['packages'][0]['id']
                                if installed['packages'][0]["built"]:
                                    if "@" not in reference:
                                        reference += "@"
                                    if self._upload_only_recipe:
                                        self._uploader.upload_recipe(reference, self._upload)
                                    else:
                                        self._uploader.upload_packages(reference,
                                                                    self._upload, package_id)
                                else:
                                    self.printer.print_message("Skipping upload for %s, "
                                                               "it hasn't been built" % package_id)


class DockerCreateRunner(object):
    def __init__(self, profile_text, base_profile_text, base_profile_name, reference,
                 conan_pip_package=None, docker_image=None, sudo_docker_command=None,
                 sudo_pip_command=False,
                 docker_image_skip_update=False, build_policy=None,
                 docker_image_skip_pull=False,
                 always_update_conan_in_docker=False,
                 upload=False, upload_retry=None, upload_only_recipe=None,
                 runner=None,
                 docker_shell="", docker_conan_home="",
                 docker_platform_param="", docker_run_options="",
                 lcow_user_workaround="",
                 test_folder=None,
                 pip_install=None,
                 docker_pip_command=None,
                 config_url=None,
                 config_args=None,
                 printer=None,
                 upload_dependencies=None,
                 conanfile=None,
                 force_selinux=None,
                 skip_recipe_export=False,
                 update_dependencies=False,
                 lockfile=None):

        self.printer = printer or Printer()
        self._upload = upload
        self._upload_retry = upload_retry
        self._upload_only_recipe = upload_only_recipe
        self._reference = reference
        self._conan_pip_package = conan_pip_package
        self._build_policy = build_policy or []
        self._docker_image = docker_image
        self._always_update_conan_in_docker = always_update_conan_in_docker
        self._docker_image_skip_update = docker_image_skip_update
        self._docker_image_skip_pull = docker_image_skip_pull
        self._sudo_docker_command = sudo_docker_command or ""
        self._sudo_pip_command = sudo_pip_command
        self._profile_text = profile_text
        self._base_profile_text = base_profile_text
        self._base_profile_name = base_profile_name
        self._docker_shell = docker_shell
        self._docker_conan_home = docker_conan_home
        self._docker_platform_param = docker_platform_param
        self._docker_run_options = docker_run_options or ""
        self._lcow_user_workaround = lcow_user_workaround
        self._runner = PrintRunner(runner, self.printer)
        self._test_folder = test_folder
        self._pip_install = pip_install
        self._docker_pip_command = docker_pip_command
        self._config_url = config_url
        self._config_args = config_args
        self._upload_dependencies = upload_dependencies or []
        self._conanfile = conanfile
        self._lockfile = lockfile
        self._force_selinux = force_selinux
        self._skip_recipe_export = skip_recipe_export
        self._update_dependencies = update_dependencies

    def _pip_update_conan_command(self):
        commands = []
        # Hack for testing when retrieving cpt from artifactory repo
        if "conan-package-tools" not in self._conan_pip_package:
            commands.append("%s %s install conan_package_tools==%s "
                            "--upgrade --no-cache" % (self._sudo_pip_command,
                                                      self._docker_pip_command,
                                                      package_tools_version))

        if self._conan_pip_package:
            commands.append("%s %s install %s --no-cache" % (self._sudo_pip_command,
                                                             self._docker_pip_command,
                                                              self._conan_pip_package))
        else:
            commands.append("%s %s install conan --upgrade --no-cache" % (self._sudo_pip_command,
                                                                          self._docker_pip_command))

        if self._pip_install:
            commands.append("%s %s install %s --upgrade --no-cache" % (self._sudo_pip_command,
                                                                       self._docker_pip_command,
                                                                       " ".join(self._pip_install)))

        command = " && ".join(commands)
        return command

    @staticmethod
    def is_selinux_running():
        if tools.which("getenforce"):
            output = subprocess.check_output("getenforce", shell=True)
            return "Enforcing" in output.decode()
        return False

    def run(self, pull_image=True, docker_entry_script=None):
        envs = self.get_env_vars()
        env_vars_text = " ".join(['-e %s="%s"' % (key, value)
                                 for key, value in envs.items() if value])

        # Run the build
        if pull_image:
            if not self._docker_image_skip_pull:
                self.pull_image()
            if not self._docker_image_skip_update and not self._always_update_conan_in_docker:
                # Update the downloaded image
                with self.printer.foldable_output("update conan"):
                    try:
                        command = '%s docker run %s --name conan_runner ' \
                                  ' %s %s %s "%s"' % (self._sudo_docker_command,
                                                   env_vars_text,
                                                   self._docker_run_options,
                                                   self._docker_image,
                                                   self._docker_shell,
                                                   self._pip_update_conan_command())

                        ret = self._runner(command)
                        if ret != 0:
                            raise Exception("Error updating the image: %s" % command)
                        # Save the image with the updated installed
                        # packages and remove the intermediate container
                        command = "%s docker commit conan_runner %s" % (self._sudo_docker_command,
                                                                        self._docker_image)
                        ret = self._runner(command)
                        if ret != 0:
                            raise Exception("Error commiting the image: %s" % command)
                    finally:
                        command = "%s docker rm conan_runner" % self._sudo_docker_command
                        ret = self._runner(command)
                        if ret != 0:
                            raise Exception("Error removing the temp container: %s" % command)

        if self._always_update_conan_in_docker:
            update_command = self._pip_update_conan_command() + " && "
        else:
            update_command = ""
        volume_options = ":z" if (DockerCreateRunner.is_selinux_running() or self._force_selinux) else ""

        command = ('%s docker run --rm -v "%s:%s/project%s" %s %s %s %s %s '
                   '"%s cd project && '
                   '%s run_create_in_docker "' % (self._sudo_docker_command,
                                                  os.getcwd(),
                                                  self._docker_conan_home,
                                                  volume_options,
                                                  env_vars_text,
                                                  self._docker_run_options,
                                                  self._docker_platform_param,
                                                  self._docker_image,
                                                  self._docker_shell,
                                                  self._lcow_user_workaround,
                                                  update_command))

        # Push entry command before to build
        if docker_entry_script:
            command = command.replace("run_create_in_docker",
                                      "%s && run_create_in_docker" % docker_entry_script)

        self.printer.print_in_docker(self._docker_image)
        ret = self._runner(command)
        if ret != 0:
            raise Exception("Error building: %s" % command)
        self.printer.print_message("Exiting docker...")

    def pull_image(self):
        with self.printer.foldable_output("docker pull"):
            ret = self._runner("%s docker pull %s" % (self._sudo_docker_command, self._docker_image))
            if ret != 0:
                raise Exception("Error pulling the image: %s" % self._docker_image)

    def get_env_vars(self):
        ret = {key: value for key, value in os.environ.items() if key.startswith("CONAN_") and
               key != "CONAN_USER_HOME"}
        ret["CONAN_REFERENCE"] = self._reference

        ret["CPT_PROFILE"] = escape_env(self._profile_text)
        ret["CPT_BASE_PROFILE"] = escape_env(self._base_profile_text)
        ret["CPT_BASE_PROFILE_NAME"] = escape_env(self._base_profile_name)

        ret["CONAN_USERNAME"] = escape_env(self._reference.user)
        ret["CONAN_TEMP_TEST_FOLDER"] = "1"  # test package folder to a temp one
        ret["CPT_UPLOAD_ENABLED"] = self._upload
        ret["CPT_UPLOAD_RETRY"] = self._upload_retry
        ret["CPT_UPLOAD_ONLY_RECIPE"] = self._upload_only_recipe
        ret["CPT_BUILD_POLICY"] = escape_env(self._build_policy)
        ret["CPT_TEST_FOLDER"] = escape_env(self._test_folder)
        ret["CPT_CONFIG_URL"] = escape_env(self._config_url)
        ret["CPT_CONFIG_ARGS"] = escape_env(self._config_args)
        ret["CPT_UPLOAD_DEPENDENCIES"] = escape_env(self._upload_dependencies)
        ret["CPT_CONANFILE"] = escape_env(self._conanfile)
        ret["CPT_LOCKFILE"] = escape_env(self._lockfile)
        ret["CPT_SKIP_RECIPE_EXPORT"] = self._skip_recipe_export
        ret["CPT_UPDATE_DEPENDENCIES"] = self._update_dependencies

        ret.update({key: value for key, value in os.environ.items() if key.startswith("PIP_")})

        return ret


def unscape_env(text):
    if not text:
        return text
    return text.replace("@@", "\n").replace('||', '"')


def escape_env(text):
    if not text:
        return text
    return text.replace("\n", "@@").replace('"', '||')


class PrintRunner(object):

    def __init__(self, runner, printer):
        self.runner = runner
        self.printer = printer

    def __call__(self, command):
        self.printer.print_command(command)
        sys.stderr.flush()
        sys.stdout.flush()
        return self.runner(command)



import os
import unittest
import sys
import json

from conans import tools
from conans.model.ref import ConanFileReference

from cpt.test.integration.base import BaseTest
from cpt.packager import ConanMultiPackager
from cpt.test.unit.utils import MockCIManager


class SimpleTest(BaseTest):

    def test_missing_full_reference(self):
        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    pass
"""
        self.save_conanfile(conanfile)
        mp = ConanMultiPackager(username="lasote")
        with self.assertRaisesRegexp(Exception, "Specify a CONAN_REFERENCE or name and version"):
            mp.add_common_builds()

    @unittest.skipUnless(sys.platform.startswith("win"), "Requires Windows")
    def test_msvc(self):
        conanfile = """from conans import ConanFile
import os

class Pkg(ConanFile):
    settings = "os", "compiler", "build_type", "arch"

    def build(self):
        assert("WindowsLibPath" in os.environ)

"""
        self.save_conanfile(conanfile)
        self.packager = ConanMultiPackager(username="lasote",
                                           channel="mychannel",
                                           visual_versions=[15],
                                           archs=["x86"],
                                           build_types=["Release"],
                                           visual_runtimes=["MD"],
                                           reference="zlib/1.2.2")
        self.packager.add_common_builds()
        self.packager.run_builds(1, 1)

    def test_msvc_exclude_precommand(self):
        conanfile = """from conans import ConanFile
import os

class Pkg(ConanFile):
    settings = "os", "compiler", "build_type", "arch"

    def build(self):
        assert("WindowsLibPath" not in os.environ)

"""
        self.save_conanfile(conanfile)
        self.packager = ConanMultiPackager(username="lasote",
                                           channel="mychannel",
                                           visual_versions=[15],
                                           archs=["x86"],
                                           build_types=["Release"],
                                           visual_runtimes=["MD"],
                                           exclude_vcvars_precommand=True,
                                           reference="zlib/1.2.2")
        self.packager.add_common_builds()
        self.packager.run_builds(1, 1)

    def test_shared_option_auto_managed(self):
        conanfile = """from conans import ConanFile

class Pkg(ConanFile):
    name = "lib"
    version = "1.0"
    settings = "os", "compiler", "build_type", "arch"
    options = {"shared": [True, False]}

"""
        self.save_conanfile(conanfile)
        self.packager = ConanMultiPackager(username="lasote")
        self.packager.add_common_builds()
        self.assertIn("lib:shared", self.packager.items[0].options)

        # Even without name and version but reference
        conanfile = """from conans import ConanFile

class Pkg(ConanFile):
    settings = "os", "compiler", "build_type", "arch"
    options = {"shared": [True, False]}


"""
        self.save_conanfile(conanfile)
        self.packager = ConanMultiPackager(username="lasote", reference="lib2/1.0")
        self.packager.add_common_builds()
        self.assertIn("lib2:shared", self.packager.items[0].options)

        self.packager = ConanMultiPackager(username="lasote", reference="lib2/1.0")
        self.packager.add_common_builds(shared_option_name=False)
        self.assertNotIn("lib2:shared", self.packager.items[0].options)

    def test_auto_managed_subdirectory(self):
        conanfile = """from conans import ConanFile

class Pkg(ConanFile):
    name = "lib"
    version = "1.0"
    settings = "os", "compiler", "build_type", "arch"
    options = {"shared": [True, False]}

"""
        cwd = os.path.join(self.tmp_folder, "subdirectory")
        tools.save(os.path.join(cwd, "conanfile.py"), conanfile)
        self.packager = ConanMultiPackager(username="lasote", cwd=cwd)
        self.packager.add_common_builds()
        self.assertGreater(len(self.packager.items), 0)
        self.assertIn("lib:shared", self.packager.items[0].options)

    def test_exported_files(self):
        conanfile = """from conans import ConanFile

class Pkg(ConanFile):
    name = "lib"
    version = "1.0"
    settings = "os"
    exports = "*"
    exports_sources = "source*"

"""
        ci_manager = MockCIManager()
        self.save_conanfile(conanfile)
        tools.save(os.path.join(self.tmp_folder, "other_file"), "Dummy contents")
        tools.save(os.path.join(self.tmp_folder, "source.cpp"), "Dummy contents")
        self.packager = ConanMultiPackager(username="lasote", reference="lib/1.0", ci_manager=ci_manager)
        self.packager.add({}, {}, {}, {})
        self.packager.run()

        ref = ConanFileReference.loads("lib/1.0@lasote/testing")
        pf = self.client_cache.package_layout(ref).export()
        found_in_export = False
        for exported in os.listdir(pf):
            if "other_file" == exported:
                found_in_export = True
                break

        self.assertTrue(found_in_export)

        pf = self.client_cache.package_layout(ref).export_sources()
        found_in_export_sources = False
        for exported in os.listdir(pf):
            if "source.cpp" == exported:
                found_in_export_sources = True
                break

        self.assertTrue(found_in_export_sources)

    def test_build_policy(self):
        ci_manager = MockCIManager()
        conanfile = """from conans import ConanFile
import os

class Pkg(ConanFile):
    name = "lib"
    version = "1.2"
    settings = "os", "compiler", "build_type", "arch"

"""
        self.save_conanfile(conanfile)
        with tools.environment_append({"CONAN_USERNAME": "lasote"}):
            self.packager = ConanMultiPackager(channel="mychannel",
                                               gcc_versions=["6"],
                                               visual_versions=["12"],
                                               archs=["x86", "x86_64"],
                                               build_types=["Release"],
                                               build_policy="outdated",
                                               ci_manager=ci_manager)
            self.packager.add_common_builds()
            self.packager.run()

        with tools.environment_append({"CONAN_USERNAME": "lasote",
                                       "CONAN_BUILD_POLICY": "outdated"}):
            self.packager = ConanMultiPackager(channel="mychannel",
                                               gcc_versions=["6"],
                                               visual_versions=["12"],
                                               archs=["x86", "x86_64"],
                                               build_types=["Release"],
                                               ci_manager=ci_manager)
            self.packager.add_common_builds()
            self.packager.run()

    def test_multiple_build_policy(self):
        ci_manager = MockCIManager()
        conanfile = """from conans import ConanFile
import os

class Pkg(ConanFile):
    name = "lib"
    version = "1.2"
    settings = "os", "compiler", "build_type", "arch"

"""
        self.save_conanfile(conanfile)
        with tools.environment_append({"CONAN_USERNAME": "lasote"}):
            self.packager = ConanMultiPackager(channel="mychannel",
                                               gcc_versions=["6"],
                                               visual_versions=["12"],
                                               archs=["x86", "x86_64"],
                                               build_types=["Release"],
                                               build_policy=["cascade", "outdated"],
                                               ci_manager=ci_manager)
            self.packager.add_common_builds()
            self.packager.run()

        with tools.environment_append({"CONAN_USERNAME": "lasote",
                                       "CONAN_BUILD_POLICY": "outdated, lib"}):
            self.packager = ConanMultiPackager(channel="mychannel",
                                               gcc_versions=["6"],
                                               visual_versions=["12"],
                                               archs=["x86", "x86_64"],
                                               build_types=["Release"],
                                               ci_manager=ci_manager)
            self.packager.add_common_builds()
            self.packager.run()

    def test_custom_conanfile(self):
        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    name = "lib"
    version = "1.2"
    settings = "os", "compiler", "build_type", "arch"
"""
        tools.save(os.path.join(self.tmp_folder, "foobar.py"), conanfile)
        with tools.environment_append({"CONAN_CONANFILE": "foobar.py"}):
            self.packager = ConanMultiPackager(username="pepe",
                                               channel="mychannel",
                                               out=self.output.write)
            self.packager.add({}, {}, {}, {})
            self.packager.run()
        self.assertIn("conanfile                 | foobar.py", self.output)

        tools.save(os.path.join(self.tmp_folder, "custom_recipe.py"), conanfile)
        self.packager = ConanMultiPackager(username="pepe",
                                           channel="mychannel",
                                           conanfile="custom_recipe.py",
                                           out=self.output.write)
        self.packager.add({}, {}, {}, {})
        self.packager.run()
        self.assertIn("conanfile                 | custom_recipe.py", self.output)

    def test_partial_reference(self):
        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    name = "foobar"
    version = "0.1.0"

    def configure(self):
        self.output.info("hello all")
"""
        tools.save(os.path.join(self.tmp_folder, "conanfile.py"), conanfile)
        with tools.environment_append({"CONAN_REFERENCE": "foobar/0.1.0@"}):
            self.packager = ConanMultiPackager(out=self.output.write)
            self.packager.add({}, {}, {}, {})
            self.packager.run()
        self.assertIn("partial_reference         | foobar/0.1.0@", self.output)

        self.packager = ConanMultiPackager(out=self.output.write)
        self.packager.add({}, {}, {}, {})
        self.packager.run()
        self.assertIn("partial_reference         | foobar/0.1.0@", self.output)

    def test_save_packages_summary(self):
        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    name = "foobar"
    version = "0.1.0"

    def configure(self):
        self.output.info("hello all")
"""
        json_file = 'cpt_summary_file.json'
        tools.save(os.path.join(self.tmp_folder, "conanfile.py"), conanfile)
        self.packager = ConanMultiPackager(out=self.output.write)
        self.packager.add({}, {}, {}, {})
        self.packager.run(summary_file=json_file)
        self.assertTrue(os.path.isfile(json_file))
        with open(json_file) as json_content:
            json_data = json.load(json_content)
            self.assertFalse(json_data[0]["package"]["error"])

        json_file = "_" + json_file
        self.packager = ConanMultiPackager(out=self.output.write)
        self.packager.add({}, {}, {}, {})
        self.packager.run()
        self.packager.save_packages_summary(json_file)
        self.assertTrue(os.path.isfile(json_file))
        with open(json_file) as json_content:
            json_data = json.load(json_content)
            self.assertFalse(json_data[0]["package"]["error"])

        json_file = "__" + json_file
        with tools.environment_append({"CPT_SUMMARY_FILE": json_file}):
            self.packager = ConanMultiPackager(out=self.output.write)
            self.packager.add({}, {}, {}, {})
            self.packager.run()
            self.assertTrue(os.path.isfile(json_file))
            with open(json_file) as json_content:
                json_data = json.load(json_content)
                self.assertFalse(json_data[0]["package"]["error"])

    def test_custom_name_version(self):
        conanfile = """from conans import ConanFile
from datetime import date
class Pkg(ConanFile):

    def configure(self):
        self.output.info("hello all")

    def set_name(self):
        self.name = "foobar"

    def set_version(self):
        today = date.today()
        self.version = today.strftime("%Y%B%d")
"""
        tools.save(os.path.join(self.tmp_folder, "conanfile.py"), conanfile)
        self.packager = ConanMultiPackager(out=self.output.write)
        self.packager.add_common_builds(pure_c=False)
        self.packager.run()

    def test_header_only_option_true(self):
        header_only = self._test_header_only(False)
        self.assertEqual(header_only, 1)
        self.packager.run()

    def test_header_only_option_false(self):
        header_only = self._test_header_only(True)
        self.assertEqual(header_only, int(len(self.packager.builds) / 2))
        self.packager.run()

    def _test_header_only(self, default_value):
        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    name = "qux"
    version = "0.1.0"
    settings = "os"
    options = {"header_only": [True, False], "shared": [True, False], "fPIC": [True, False]}
    default_options = {"header_only": %s, "shared": False, "fPIC": True}

    def configure(self):
        if self.options.header_only:
            del self.options.shared
            del self.options.fPIC

    def package_id(self):
        if self.options.header_only:
            self.info.header_only()
""" % default_value
        tools.save(os.path.join(self.tmp_folder, "conanfile.py"), conanfile)
        self.packager = ConanMultiPackager(out=self.output.write)
        self.packager.add_common_builds(pure_c=False)

        header_only = 0
        for build in self.packager.builds:
            _, options, _, _ = build
            if options.get("qux:header_only") == (not default_value):
                header_only += 1
        return header_only

    def test_build_all_option_values(self):
        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    name = "qux"
    version = "0.1.0"
    options = {"shared": [True, False], "fPIC": [True, False],
               "header_only": [True, False], "foo": [True, False],
               "bar": ["baz", "qux", "foobar"], "blah": "ANY"}
    default_options = {"shared": False, "fPIC": True, "header_only": False,
                       "foo": False, "bar": "baz", "blah": 42}

    def configure(self):
        self.output.info("hello all")
"""
        tools.save(os.path.join(self.tmp_folder, "conanfile.py"), conanfile)
        self.packager = ConanMultiPackager(out=self.output.write)
        self.packager.add_common_builds(pure_c=False, build_all_options_values=["qux:foo", "qux:bar", "qux:blah"])
        self.packager.run()